@@ -21,12 +21,6 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/common/events.h $
- | @last_commit: $Date: 2015-04-20 16:32:24 +0200 (lun, 20 abr 2015) $
- | @version:     $Revision: 3250 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
-
 #ifndef __EVENTS_H_INCLUDED__
 #define __EVENTS_H_INCLUDED__
 
@@ -36,10 +30,8 @@
 extern "C" {
 #endif
 unsigned IsMPI (unsigned EvType);
-unsigned IsPACX (unsigned EvType);
 unsigned IsOpenMP (unsigned EvType);
 unsigned IsMISC (unsigned EvType);
-unsigned IsTRT (unsigned EvType);
 
 unsigned IsBurst (unsigned EvType);
 unsigned IsHwcChange (unsigned EvType);
@@ -82,12 +74,16 @@ unsigned IsMPICollective (unsigned EvType);
 #define SAMPLING_LINE_EV         30000100
 #define SAMPLING_CALLER_OFFSET     100000
 #define HWC_SET_OVERFLOW_EV      31000000
-#define SAMPLING_ADDRESS_EV                  32000000
-#define SAMPLING_ADDRESS_MEM_LEVEL_EV        32000001
-#define SAMPLING_ADDRESS_MEM_HITORMISS_EV    32000002
-#define SAMPLING_ADDRESS_TLB_LEVEL_EV        32000003
-#define SAMPLING_ADDRESS_TLB_HITORMISS_EV    32000004
-#define SAMPLING_ADDRESS_REFERENCE_COST_EV   32000005
+#define SAMPLING_ADDRESS_LD_EV               32000000
+#define SAMPLING_ADDRESS_ST_EV               32000001
+#define SAMPLING_ADDRESS_MEM_LEVEL_EV        32000002
+#define SAMPLING_ADDRESS_MEM_HITORMISS_EV    32000003
+#define SAMPLING_ADDRESS_TLB_LEVEL_EV        32000004
+#define SAMPLING_ADDRESS_TLB_HITORMISS_EV    32000005
+#define SAMPLING_ADDRESS_REFERENCE_COST_EV   32000006
+#define SAMPLING_ADDRESS_ALLOCATED_OBJECT_EV 32000007
+#define SAMPLING_ADDRESS_STATIC_OBJECT_EV    32000008
+#define SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV 32000100 /* internal purposes, not emitted into paraver tracefile */
 
 #define APPL_EV                  40000001
 #define TRACE_INIT_EV            40000002
@@ -103,13 +99,13 @@ unsigned IsMPICollective (unsigned EvType);
 #define HWC_EV                   40000009
 #define IO_DESCRIPTOR_EV         40000010
 #define IO_SIZE_EV               40000011
+#define IO_DESCRIPTOR_TYPE_EV    40000013
 #define TRACING_EV               40000012
 #define SET_TRACE_EV             40000014
 #define CPU_BURST_EV             40000015
 #define RUSAGE_EV                40000016
 #define MPI_STATS_EV             40000017
 #define TRACING_MODE_EV          40000018
-#define PACX_STATS_EV            40000019
 #define MEMUSAGE_EV              40000020
 #define USER_SEND_EV             40000021
 #define USER_RECV_EV             40000022
@@ -139,6 +135,7 @@ unsigned IsMPICollective (unsigned EvType);
 #define DYNAMIC_MEM_REQUESTED_SIZE_EV      DYNAMIC_MEM_EV+1  /* Used in merger only */
 #define DYNAMIC_MEM_POINTER_IN_EV          DYNAMIC_MEM_EV+2  /* Used in merger only, free input, realloc in */
 #define DYNAMIC_MEM_POINTER_OUT_EV         DYNAMIC_MEM_EV+3  /* Used in merger only, malloc output, calloc output, realloc out */
+#define CLOCK_FROM_SYSTEM_EV     40000050
 
 #define ADDRESSES_FOR_BINARY_EV  41000000
 
@@ -173,6 +170,11 @@ enum {
    MEMUSAGE_EVENTS_COUNT /* Total number of memusage events */
 };
 
+#define JAVA_JVMTI_GARBAGECOLLECTOR_EV     48000001
+#define JAVA_JVMTI_EXCEPTION_EV            48000002
+#define JAVA_JVMTI_OBJECT_ALLOC_EV         48000003
+#define JAVA_JVMTI_OBJECT_FREE_EV          48000004
+
 #define OMP_STATS_BASE           65000000
 enum {
 	OMP_NUM_TASKS_INSTANTIATED = 0,
@@ -203,18 +205,6 @@ enum {
    MPI_STATS_EVENTS_COUNT /* Total number of MPI statistics */
 };
 
-#define PACX_STATS_BASE           55000000
-enum {
-   PACX_STATS_P2P_COMMS_EV = 0,
-   PACX_STATS_P2P_BYTES_SENT_EV,
-   PACX_STATS_P2P_BYTES_RECV_EV,
-   PACX_STATS_GLOBAL_COMMS_EV,
-   PACX_STATS_GLOBAL_BYTES_SENT_EV,
-   PACX_STATS_GLOBAL_BYTES_RECV_EV,
-   PACX_STATS_TIME_IN_PACX_EV, 
-   PACX_STATS_EVENTS_COUNT /* Total number of PACX statistics */
-};
-
 #define FUNCT_BASE               41000000
 #define FUNCT_MAX                    1000
 
@@ -314,6 +304,7 @@ enum {
 #define MPI_FILE_READ_AT_ALL_EV      50000107
 #define MPI_FILE_WRITE_AT_EV         50000108
 #define MPI_FILE_WRITE_AT_ALL_EV     50000109
+#define MPI_IO_SIZE_EV               50000110
 
 #define MPI_GET_EV                   50000200
 #define MPI_PUT_EV                   50000201
@@ -325,102 +316,28 @@ enum {
 #define MPI_WIN_COMPLETE_EV          50000207
 #define MPI_WIN_WAIT_EV              50000208
 
+#define MPI_IREDUCE_EV               50000210
+#define MPI_IALLREDUCE_EV            50000211
+#define MPI_IBARRIER_EV              50000212
+#define MPI_IBCAST_EV                50000213
+#define MPI_IALLTOALL_EV             50000214
+#define MPI_IALLTOALLV_EV            50000215
+#define MPI_IALLGATHER_EV            50000216
+#define MPI_IALLGATHERV_EV           50000217
+#define MPI_IGATHER_EV               50000218
+#define MPI_IGATHERV_EV              50000219
+#define MPI_ISCATTER_EV              50000220
+#define MPI_ISCATTERV_EV             50000221
+#define MPI_IREDUCESCAT_EV           50000222
+#define MPI_ISCAN_EV                 50000223
+
+
 
 #define MPI_IPROBE_COUNTER_EV        50000300
 #define MPI_TIME_OUTSIDE_IPROBES_EV  50000301
 #define MPI_REQUEST_GET_STATUS_COUNTER_EV               50000302
 #define MPI_TIME_OUTSIDE_MPI_REQUEST_GET_STATUS_EV      50000303
 
-/******************************************************************************
- *   User events to trace several MPI functions.
- *   MUST be between 50000001 - 50999999
- ******************************************************************************/
-#define PACX_MIN_EV                   PACX_INIT_EV
-#define PACX_MAX_EV                   51999999
-
-#define PACX_INIT_EV                  51000001
-#define PACX_BSEND_EV                 51000002
-#define PACX_SSEND_EV                 51000003
-#define PACX_BARRIER_EV               51000004
-#define PACX_BCAST_EV                 51000005
-#define PACX_SEND_EV                  51000018
-#define PACX_SENDRECV_EV              51000017
-#define PACX_SENDRECV_REPLACE_EV      51000081
-#define PACX_RECV_EV                  51000019
-#define PACX_IBSEND_EV                51000020
-#define PACX_ISSEND_EV                51000021
-#define PACX_ISEND_EV                 51000022
-#define PACX_IRECV_EV                 51000023
-#define PACX_IRCV_EV                  51000025
-#define PACX_TEST_EV                  51000026
-#define PACX_TESTALL_EV               51000082
-#define PACX_TESTANY_EV               51000083
-#define PACX_TESTSOME_EV              51000084
-#define PACX_TEST_COUNTER_EV          51000080
-#define PACX_WAIT_EV                  51000027
-#define PACX_CANCEL_EV                51000030
-#define PACX_RSEND_EV                 51000031
-#define PACX_IRSEND_EV                51000032
-#define PACX_ALLTOALL_EV              51000033
-#define PACX_ALLTOALLV_EV             51000034
-#define PACX_ALLREDUCE_EV             51000035
-#define PACX_REDUCE_EV                51000038
-#define PACX_WAITALL_EV               51000039
-#define PACX_WAITANY_EV               51000068
-#define PACX_WAITSOME_EV              51000069
-#define PACX_IRECVED_EV               51000040
-#define PACX_GATHER_EV                51000041
-#define PACX_GATHERV_EV               51000042
-#define PACX_SCATTER_EV               51000043
-#define PACX_SCATTERV_EV              51000044
-#define PACX_FINALIZE_EV              51000045
-#define PACX_COMM_RANK_EV             51000046
-#define PACX_COMM_SIZE_EV             51000047
-#define PACX_COMM_CREATE_EV           51000048
-#define PACX_COMM_DUP_EV              51000049
-#define PACX_COMM_SPLIT_EV            51000050
-#define PACX_RANK_CREACIO_COMM_EV     51000051      /* Used to define communicators */
-#define PACX_ALIAS_COMM_CREATE_EV     50000061      /* Used to define communicators */
-#define PACX_ALLGATHER_EV             51000052
-#define PACX_ALLGATHERV_EV            51000053
-#define PACX_CART_CREATE_EV           51000058
-#define PACX_CART_SUB_EV              51000059
-#define PACX_CART_COORDS_EV           51000060
-#define PACX_REDUCESCAT_EV            51000062
-#define PACX_SCAN_EV                  51000063
-#define PACX_PROBE_EV                 51000065
-#define PACX_IPROBE_EV                51000066
-#define PACX_COMM_FREE_EV             51000067
-
-#define PACX_PERSIST_REQ_EV           51000070
-#define PACX_START_EV                 51000071
-#define PACX_STARTALL_EV              51000072
-#define PACX_REQUEST_FREE_EV          51000073
-#define PACX_RECV_INIT_EV             51000074
-#define PACX_SEND_INIT_EV             51000075
-#define PACX_BSEND_INIT_EV            51000076
-#define PACX_RSEND_INIT_EV            51000077
-#define PACX_SSEND_INIT_EV            51000078
-
-#define PACX_GLOBAL_OP_SENDSIZE       (PACX_INIT_EV+100000)
-#define PACX_GLOBAL_OP_RECVSIZE       (PACX_INIT_EV+100001)
-#define PACX_GLOBAL_OP_ROOT           (PACX_INIT_EV+100002)
-#define PACX_GLOBAL_OP_COMM           (PACX_INIT_EV+100003)
-
-#define PACX_FILE_OPEN_EV             51000100
-#define PACX_FILE_CLOSE_EV            51000101
-#define PACX_FILE_READ_EV             51000102
-#define PACX_FILE_READ_ALL_EV         51000103
-#define PACX_FILE_WRITE_EV            51000104
-#define PACX_FILE_WRITE_ALL_EV        51000105
-#define PACX_FILE_READ_AT_EV          51000106
-#define PACX_FILE_READ_AT_ALL_EV      51000107
-#define PACX_FILE_WRITE_AT_EV         51000108
-#define PACX_FILE_WRITE_AT_ALL_EV     51000109
-
-#define PACX_IPROBE_COUNTER_EV        51000300
-#define PACX_TIME_OUTSIDE_IPROBES_EV  51000301
-
 /******************************************************************************
  *   User events for BG PERSONALITY
  ******************************************************************************/
@@ -495,7 +412,7 @@ enum {
 #define OMP_STATS_EV             60000060
 
 /******************************************************************************
- *   User events to trace Pthread/TRT parallel execution.
+ *   User events to trace Pthread parallel execution.
  ******************************************************************************/
 #define PTHREAD_BASE_EV            61000000
 #define PTHREAD_EXIT_EV            61000001
@@ -515,11 +432,6 @@ enum {
 #define PTHREAD_FUNC_EV          60000020
 #define PTHREAD_FUNC_LINE_EV     60000120
 
-#define TRT_SPAWN_EV             62000001
-#define TRT_READ_EV              62000002
-#define TRT_USRFUNC_EV           62000003
-#define TRT_USRFUNC_LINE_EV      62000003
-
 #define CUDACALL_EV              63000001
 #define CUDAMEMCPY_SIZE_EV       63000002
 #define CUDAFUNC_EV              63000019
@@ -540,6 +452,7 @@ enum {
 #define CUDAMEMCPY_GPU_EV        63200003
 #define CUDATHREADBARRIER_GPU_EV 63200004
 #define CUDAMEMCPYASYNC_GPU_EV   63200007
+/* To associate stream with Paraver thread */
 #define CUDASTREAMBARRIER_THID_EV 63300000
 
 #define OPENCL_KERNEL_NAME_EV                         64200000
@@ -623,6 +536,9 @@ enum {
 #define OPENCL_CLENQUEUEREADBUFFERRECT_ASYNC_ACC_EV   64100050
 #define OPENCL_CLENQUEUEWRITEBUFFER_ASYNC_ACC_EV      64100051
 #define OPENCL_CLENQUEUEWRITEBUFFERRECT_ASYNC_ACC_EV  64100052
+/* To associate command queue with Paraver thread */
+#define OPENCL_CLFINISH_THID_EV                       64300000
+
 
 #define CALLER_EV                70000000
 #define CALLER_LINE_EV           80000000
@@ -927,14 +843,24 @@ typedef enum
 	MISC_TYPE,
 	OPENMP_TYPE,
 	PTHREAD_TYPE,
-	TRT_TYPE,
-	PACX_TYPE,
-	PACX_COMM_ALIAS_TYPE,
 	CUDA_TYPE,
 	OPENCL_TYPE,
-	OPENSHMEM_TYPE
+	OPENSHMEM_TYPE,
+	JAVA_TYPE
 } EventType_t;
 
+
+/* File descriptor classifications */
+
+typedef enum
+{
+	DESCRIPTOR_TYPE_UNKNOWN = 0, /* unknown */
+	DESCRIPTOR_TYPE_REGULARFILE, /* regular file */
+	DESCRIPTOR_TYPE_SOCKET,      /* socket */
+	DESCRIPTOR_TYPE_FIFO_PIPE,   /* fifo or pipe */
+	DESCRIPTOR_TYPE_ATTY         /* connected to the terminal? */
+} ExtraeDescriptorType_t;
+
 EventType_t getEventType (unsigned EvType, unsigned *Type);
 
 #endif /* __EVENTS_H_INCLUDED__ */
