@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/common/mpi2out.c $
- | @last_commit: $Date: 2015-04-23 15:48:32 +0200 (jue, 23 abr 2015) $
- | @version:     $Revision: 3267 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: mpi2out.c 3267 2015-04-23 13:48:32Z harald $";
-
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -111,8 +104,8 @@ void Help (const char *ProgName)
 		  "    -o file              Output trace file name.\n"
 		  "    -e file              Uses the executable file to obtain some information.\n"
 		  "    -f file              MpitFILE File with the names of the \".mpit\" input files.\n"
-		  "    -syn                 Synchronize traces at the end of MPI_Init.\n"
-		  "    -syn-node            Synchronize traces using node information.\n"
+		  "    -syn                 Synchronize traces at the MPI task-level using the MPI_Init information.\n"
+		  "    -syn-node            Synchronize traces at the MPI node-level using the MPI_Init information.\n"
 		  "    -no-syn              Do not synchronize traces at the end of MPI_Init.\n"
 		  "    -maxmem M            Uses up to M megabytes of memory at the last step of merging process.\n"
 		  "    -dimemas             Force the generation of a Dimemas trace.\n"
@@ -134,7 +127,7 @@ void Help (const char *ProgName)
 #endif
 		  "    -s file              Indicates the symbol (*.sym) file attached to the *.mpit files.\n"
 		  "    -d/-dump             Sequentially dumps the contents of every *.mpit file.\n"
-		  "    -dump-without-time   Do not show event time in when dumping events.\n"
+		  "    -dump-without-time   Do not show event time in when dumping events (valuable for test purposes).\n"
 		  "    -remove-files        Remove intermediate files after processing them.\n"
 		  "    -split-states        Do not merge consecutives states that are the same.\n"
 		  "    -skip-sendrecv       Do not emit communication for SendReceive operations.\n"
@@ -144,6 +137,7 @@ void Help (const char *ProgName)
 		  "    -emit-library-events Emit library information for unknown references if possible.\n"
 		  "    -sort-addresses      Sort file name, line events in information linked with source code.\n"
 		  "    -task-view           Swap the thread level in Paraver timeline to show Nanos Tasks.\n"
+          "    -without-addresses   Do not emit address information into PCF (valuable for test purposes).\n"
 		  "    --                   Take the next trace files as a diferent parallel task.\n"
 		  "\n",
           ProgName, ProgName, ProgName);
@@ -154,7 +148,7 @@ void Help (const char *ProgName)
  ***  Adds an MPIT file into the required structures.
  ******************************************************************************/
 
-static void Process_MPIT_File (char *file, char *node, char *thdname, int *cptask,
+static void Process_MPIT_File (char *file, char *thdname, int *cptask,
 	int taskid)
 {
 	int name_length;
@@ -163,14 +157,17 @@ static void Process_MPIT_File (char *file, char *node, char *thdname, int *cptas
 	int i;
 	int cur_ptask = *cptask;
 	char *tmp_name;
+	size_t pos;
+	int has_node_separator;
+	int hostname_len;
 
-        xrealloc(InputTraces, InputTraces, sizeof(struct input_t) * (nTraces + 1));
-        if (InputTraces == NULL)
-        {
-          perror ("realloc");
-          fprintf (stderr, "mpi2prv: Cannot allocate InputTraces memory for MPIT %d. Dying...\n", nTraces + 1);
-          exit (1);
-        }
+	xrealloc(InputTraces, InputTraces, sizeof(struct input_t) * (nTraces + 1));
+	if (InputTraces == NULL)
+	{
+		perror ("realloc");
+		fprintf (stderr, "mpi2prv: Cannot allocate InputTraces memory for MPIT %d. Dying...\n", nTraces + 1);
+		exit (1);
+	}
 
 	InputTraces[nTraces].InputForWorker = -1;
 	InputTraces[nTraces].name = (char *) malloc (strlen (file) + 1);
@@ -182,18 +179,41 @@ static void Process_MPIT_File (char *file, char *node, char *thdname, int *cptas
 	}
 	strcpy (InputTraces[nTraces].name, file);
 
-	if (node != NULL)
+	pos = strlen(file)-strlen(EXT_MPIT)-DIGITS_PID-DIGITS_TASK
+	  -DIGITS_THREAD-1; // Last -1 is for extra .
+	has_node_separator = FALSE;
+	hostname_len = 0;
+	while (!has_node_separator)
 	{
-		InputTraces[nTraces].node = strdup (node);
-		if (InputTraces[nTraces].node == NULL)
+		has_node_separator = file[pos] == TEMPLATE_NODE_SEPARATOR_CHAR;
+		if (has_node_separator)
 		{
-			fprintf (stderr, "mpi2prv: Error cannot obtain memory for NODE information!\n");
-			fflush (stderr);
-			exit (1);
+			InputTraces[nTraces].node = (char*) malloc (
+			  (hostname_len+1)*sizeof(char));
+			if (InputTraces[nTraces].node == NULL)
+			{
+				fprintf (stderr, "mpi2prv: Error cannot obtain memory for NODE information!\n");
+				fflush (stderr);
+				exit (1);
+			}
+			snprintf (InputTraces[nTraces].node, hostname_len, "%s", &file[pos+1]);
+			break;
+		}
+		else
+		{
+			if (pos == 0)
+			{
+				fprintf (stderr, "merger: Could not find node separator in file '%s'\n", file);
+				InputTraces[nTraces].node = "(unknown)";
+				break;
+			}
+			else
+			{
+				hostname_len++;
+				pos--;
+			}
 		}
 	}
-	else 
-		InputTraces[nTraces].node = "(unknown)";
 
 	name_length = strlen (InputTraces[nTraces].name);
 	tmp_name = InputTraces[nTraces].name;
@@ -240,6 +260,8 @@ static void Process_MPIT_File (char *file, char *node, char *thdname, int *cptas
 	InputTraces[nTraces].thread++;
 	InputTraces[nTraces].ptask = cur_ptask;
 	InputTraces[nTraces].order = nTraces;
+	/* This will be changed latter if Read_SPAWN_file is applied */
+	InputTraces[nTraces].SpawnOffset = 0;
 
 	if (thdname != NULL)
 	{
@@ -277,31 +299,6 @@ static void Process_MPIT_File (char *file, char *node, char *thdname, int *cptas
 	nTraces++;
 }
 
-/******************************************************************************
- **      Function name : strip
- **      Author : HSG
- **      Description : Removes spaces and control characters from a string.
- ******************************************************************************/
-
-static char *strip (char *buffer)
-{
-	int l = strlen (buffer);
-	int min = 0, max = l - 1;
-
-	if (min == max)
-		return NULL;
-
-	while (isspace (buffer[min]) || iscntrl (buffer[min]))
-		min++;
-
-	while (isspace (buffer[max]) || iscntrl (buffer[min]))
-		max--;
-
-	buffer[max + 1] = (char) 0;
-
-	return &(buffer[min]);
-}
-
 #if defined(MPI_SUPPORTS_MPI_COMM_SPAWN)
 void Read_SPAWN_file (char *mpit_file, int current_ptask)
 {
@@ -355,7 +352,6 @@ void Read_MPITS_file (const char *file, int *cptask, FileOpen_t opentype, int ta
 {
 	int info;
 	char mybuffer[4096];
-	char host[2048];
 	char thdname[2048];
 	char path[2048];
 	FILE *fd = fopen (file, "r");
@@ -384,10 +380,10 @@ void Read_MPITS_file (const char *file, int *cptask, FileOpen_t opentype, int ta
 		{
 			char *stripped;
 
-			path[0] = host[0] = thdname[0] = (char) 0;
+			path[0] = thdname[0] = (char) 0;
 
-			info = sscanf (mybuffer, "%s on %s named %s", path, host, thdname);
-			stripped = strip (path);
+			info = sscanf (mybuffer, "%s named %s", path, thdname);
+			stripped = trim (path);
 
 			if (strncmp (mybuffer, "--", 2) == 0)
 			{
@@ -413,22 +409,22 @@ void Read_MPITS_file (const char *file, int *cptask, FileOpen_t opentype, int ta
 							char *directory = dirname (duplicate);
 
 							sprintf (dir_file, "%s%s", directory, stripped_basename);
-							Process_MPIT_File (dir_file, (info>=2)?host:NULL, (info==3)?thdname:NULL, cptask, taskid);
+							Process_MPIT_File (dir_file, (info==2)?thdname:NULL, cptask, taskid);
 
 							free (duplicate);
 						}
 						else
-							Process_MPIT_File (&stripped_basename[1], (info>=2)?host:NULL, (info==3)?thdname:NULL, cptask, taskid);
+							Process_MPIT_File (&stripped_basename[1], (info==2)?thdname:NULL, cptask, taskid);
 					}
 					else
 						fprintf (stderr, "merger: Error cannot find 'set-' signature in filename %s\n", stripped);
 				}
 				else
-					Process_MPIT_File (stripped, (info>=2)?host:NULL, (info==3)?thdname:NULL, cptask, taskid);
+					Process_MPIT_File (stripped, (info==2)?thdname:NULL, cptask, taskid);
 			}
 			else if (info >= 1 && opentype == FileOpen_Absolute)
 			{
-				Process_MPIT_File (stripped, (info>=2)?host:NULL, (info==3)?thdname:NULL, cptask, taskid);
+				Process_MPIT_File (stripped, (info==2)?thdname:NULL, cptask, taskid);
 			}
 			else if (info >= 1 && opentype == FileOpen_Relative)
 			{
@@ -444,12 +440,12 @@ void Read_MPITS_file (const char *file, int *cptask, FileOpen_t opentype, int ta
 						char *directory = dirname (duplicate);
 
 						sprintf (dir_file, "%s%s", directory, stripped_basename);
-						Process_MPIT_File (dir_file, (info>=2)?host:NULL, (info==3)?thdname:NULL, cptask, taskid);
+						Process_MPIT_File (dir_file, (info==2)?thdname:NULL, cptask, taskid);
 
 						free (duplicate);
 					}
 					else
-						Process_MPIT_File (&stripped_basename[1], (info>=2)?host:NULL, (info==3)?thdname:NULL, cptask, taskid);
+						Process_MPIT_File (&stripped_basename[1], (info==2)?thdname:NULL, cptask, taskid);
 				}
 				else
 					fprintf (stderr, "merger: Error cannot find 'set-' signature in filename %s\n", stripped);
@@ -802,6 +798,16 @@ void ProcessArgs (int rank, int argc, char *argv[])
 			set_option_dump_Time (TRUE);
 			continue;
 		}
+		if (!strcmp (argv[CurArg], "-with-addresses"))
+		{
+			set_option_dump_Addresses (TRUE);
+			continue;
+		}
+		if (!strcmp (argv[CurArg], "-without-addresses"))
+		{
+			set_option_dump_Addresses (FALSE);
+			continue;
+		}
 		if (!strcmp (argv[CurArg], "-maxmem"))
 		{
 
@@ -899,7 +905,7 @@ void ProcessArgs (int rank, int argc, char *argv[])
 			continue;
 		}
 		else
-			Process_MPIT_File ((char *) (argv[CurArg]), NULL, NULL, &cur_ptask, rank);
+			Process_MPIT_File (argv[CurArg], NULL, &cur_ptask, rank);
 	}
 	set_option_merge_NumApplications (cur_ptask);
 
@@ -1390,7 +1396,7 @@ int merger_post (int numtasks, int taskid)
 	if (taskid == 0)
 	{
 		fprintf (stdout, "mpi2prv: Checking for target directory existance...");
-		char *dirn = dirname(strdup(strip(get_merge_OutputTraceName())));
+		char *dirn = dirname(strdup(trim(get_merge_OutputTraceName())));
 		if (!directory_exists(dirn))
 		{
 			fprintf (stdout, " does not exist. Creating ...");
@@ -1407,11 +1413,11 @@ int merger_post (int numtasks, int taskid)
 	}
 
 	if (get_option_merge_ParaverFormat())
-		error = Paraver_ProcessTraceFiles (strip(get_merge_OutputTraceName()),
-			nTraces, InputTraces, get_option_merge_NumApplications(),
+		error = Paraver_ProcessTraceFiles (nTraces, InputTraces,
+		    get_option_merge_NumApplications(),
 			NodeCPUinfo, numtasks, taskid);
 	else
-		error = Dimemas_ProcessTraceFiles (strip(get_merge_OutputTraceName()),
+		error = Dimemas_ProcessTraceFiles (trim(get_merge_OutputTraceName()),
 			nTraces, InputTraces, get_option_merge_NumApplications(),
 			NodeCPUinfo, numtasks, taskid);
 
