@@ -21,14 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/file_set.c $
- | @last_commit: $Date: 2015-04-23 17:12:21 +0200 (jue, 23 abr 2015) $
- | @version:     $Revision: 3269 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
-
-static char UNUSED rcsid[] = "$Id: file_set.c 3269 2015-04-23 15:12:21Z harald $";
+#include <errno.h>
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
@@ -66,6 +60,9 @@ static char UNUSED rcsid[] = "$Id: file_set.c 3269 2015-04-23 15:12:21Z harald $
 # include "mpi-aux.h"
 # include "tree-logistics.h"
 #endif
+#if defined(HAVE_SIONLIB)
+# include "sion.h"
+#endif
 #include "labels.h"
 #include "utils.h"
 #include "semantics.h"
@@ -227,6 +224,44 @@ static int AddFile_FS (FileItem_t * fitem, struct input_t *IFile, int taskid)
 	event_t *ptr_last = NULL;
 
 	strcpy (trace_file_name, IFile->name);
+#if defined(HAVE_SIONLIB)
+	int rc, sid;
+	int ntasks, nfiles;
+	FILE *fp;
+	sion_int64 *chunksizes = NULL; 
+	sion_int32 fsblksize;
+	int *globalranks = NULL; // allocated by sion_open
+
+	// default is ANSI-C
+	sid = sion_open("data.mpit", "rb", &ntasks, &nfiles, &chunksizes,
+		&fsblksize, &globalranks, &fp);
+	if (sid == -1)
+	{
+		perror ("sion_open");
+		fprintf (stderr, "mpi2prv Error: Failed to open trace-file\n");
+	}
+
+	int size, block;
+	sion_int64 globalskip;
+	sion_int64 start_of_varheader;
+	sion_int64 *sion_chunksizes;
+	sion_int64 *sion_globalranks;
+	sion_int64 *sion_blockcount;
+	sion_int64 *sion_blocksizes;
+
+	sion_get_locations(sid, &size, &block, &globalskip, &start_of_varheader,
+	  &sion_chunksizes, &sion_globalranks, &sion_blockcount, &sion_blocksizes);
+
+	int rank; 
+	int blksize; 
+	int blknum;
+
+	for (blknum = 0; blknum < sion_blockcount[taskid]; blknum++)
+		blksize = sion_blocksizes[size * blknum + IFile->task -1];
+	//fprintf(stdout, "rank %d byes %d\n", IFile->task - 1, blksize);
+
+	fd_trace = fp;
+#else
 	fd_trace = fopen (trace_file_name, "r");
 	if (NULL == fd_trace)
 	{
@@ -234,6 +269,7 @@ static int AddFile_FS (FileItem_t * fitem, struct input_t *IFile, int taskid)
 		fprintf (stderr, "mpi2prv Error: Opening trace file %s\n", trace_file_name);
 		return (-1);
 	}
+#endif
 
 #if defined(SAMPLING_SUPPORT)
 	strcpy (sample_file_name, IFile->name);
@@ -249,7 +285,9 @@ static int AddFile_FS (FileItem_t * fitem, struct input_t *IFile, int taskid)
 
 	fd_online = open (online_file_name, O_RDONLY);
 #endif
-
+#if defined(HAVE_SIONLIB)
+	trace_file_size = blksize;
+#else
 	ret = fseeko (fd_trace, 0, SEEK_END);
 	if (0 != ret)
 	{
@@ -257,7 +295,7 @@ static int AddFile_FS (FileItem_t * fitem, struct input_t *IFile, int taskid)
 		exit (1);
 	}
 	trace_file_size = ftello (fd_trace);
-
+#endif
 #if defined(SAMPLING_SUPPORT)
 	if (NULL != fd_sample)
 	{
@@ -291,7 +329,10 @@ static int AddFile_FS (FileItem_t * fitem, struct input_t *IFile, int taskid)
 #endif
 	fitem->num_of_events = (fitem->size>0)?fitem->size/sizeof(event_t):0;
 
+#if !defined(HAVE_SIONLIB)
 	rewind (fd_trace);
+#endif
+
 #if defined(SAMPLING_SUPPORT)
 	if (NULL != fd_sample)
 		rewind (fd_sample);
@@ -575,6 +616,8 @@ PRVFileSet_t * Map_Paraver_files (FileSet_t * fset,
 
 	*num_of_events = total;
 
+	prvfset->SkipAsMasterOfSubtree = FALSE;
+
 	return prvfset;
 }
 
@@ -1111,27 +1154,6 @@ event_t *Search_MPI_IRECVED (event_t * current, long long request, FileItem_t *
 	return NULL;
 }
 
-/******************************************************************************
- ***  Search_PACX_IRECVED
- ******************************************************************************/
-event_t *Search_PACX_IRECVED (event_t * current, long long request, FileItem_t * freceive)
-{
-	event_t *irecved = current;
-
-	freceive->tmp = irecved;
-	/* freceive->tmp = freceive->first; */
-
-	if (Get_EvEvent (irecved) == PACX_IRECVED_EV)
-		if (Get_EvAux (irecved) == request)
-			return irecved;
-
-	while ((irecved = NextRecvG_FS (freceive)) != NULL)
-		if (Get_EvEvent (irecved) == PACX_IRECVED_EV)
-			if (Get_EvAux (irecved) == request)
-				return irecved;
-	return NULL;
-}
-
 void Rewind_FS (FileSet_t * fs)
 {
 	unsigned int i;
@@ -1305,30 +1327,25 @@ int Search_Synchronization_Times (int taskid, int ntasks, FileSet_t * fset,
 			{
 				FileItem_t *fi = &(fset->files[i]);
 
-				UINT64 mpi_init_end_time, pacx_init_end_time, trace_init_end_time, shmem_init_end_time;
-				int found_mpi_init_end_time, found_pacx_init_end_time, found_trace_init_end_time, found_shmem_init_end_time;
+				UINT64 mpi_init_end_time, trace_init_end_time, shmem_init_end_time;
+				int found_mpi_init_end_time, found_trace_init_end_time, found_shmem_init_end_time;
 
-				found_mpi_init_end_time = found_pacx_init_end_time = found_trace_init_end_time = found_shmem_init_end_time = FALSE;
-				mpi_init_end_time = pacx_init_end_time = trace_init_end_time = shmem_init_end_time = 0;
+				found_mpi_init_end_time = found_trace_init_end_time = found_shmem_init_end_time = FALSE;
+				mpi_init_end_time = trace_init_end_time = shmem_init_end_time = 0;
 
 				/* Save the starting tracing time of this task */
 				StartingTimes[fi->mpit_id] = current->time;
 
-				/* Locate MPI_INIT_EV, PACX_INIT_EV and/or TRACE_INIT_EV
-				   Be careful not to stop on TRACE_INIT_EV because a MPI_INIT_EV/PACX_INIT_EV may
+				/* Locate MPI_INIT_EV or TRACE_INIT_EV
+				   Be careful not to stop on TRACE_INIT_EV because a MPI_INIT_EV may
 				   appear in future and they're always synchronized, not as TRACE_INIT_EV */
-				while (current != NULL && !found_mpi_init_end_time && !found_pacx_init_end_time)
+				while (current != NULL && !found_mpi_init_end_time)
 				{
 					if (Get_EvEvent(current) == MPI_INIT_EV && Get_EvValue(current) == EVT_END)
 					{
 						mpi_init_end_time = Get_EvTime(current);
 						found_mpi_init_end_time = TRUE;
 					}
-					else if (Get_EvEvent(current) == PACX_INIT_EV && Get_EvValue(current) == EVT_END)
-					{
-						pacx_init_end_time = Get_EvTime(current);
-						found_pacx_init_end_time = TRUE;
-					}
 					else if (Get_EvEvent(current) == TRACE_INIT_EV && Get_EvValue(current) == EVT_END)
 					{
 						trace_init_end_time = Get_EvTime(current);
@@ -1345,8 +1362,6 @@ int Search_Synchronization_Times (int taskid, int ntasks, FileSet_t * fset,
 
 				if (found_mpi_init_end_time)
 					SynchronizationTimes[fi->mpit_id] = mpi_init_end_time;
-				else if (found_pacx_init_end_time)
-					SynchronizationTimes[fi->mpit_id] = pacx_init_end_time;
 				else if (found_trace_init_end_time)
 					SynchronizationTimes[fi->mpit_id] = trace_init_end_time;
 				else if (found_shmem_init_end_time)
@@ -1611,7 +1626,6 @@ long long GetTraceOptions (FileSet_t * fset, int numtasks, int taskid)
 	while (current != NULL)
 	{
 		if ((Get_EvEvent (current) == MPI_INIT_EV && Get_EvValue (current) == EVT_END) ||
-		    (Get_EvEvent (current) == PACX_INIT_EV && Get_EvValue (current) == EVT_END) ||
 		    (Get_EvEvent (current) == TRACE_INIT_EV && Get_EvValue (current) == EVT_END))
 		{
 			options = Get_EvAux(current);
