@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/labels.c $
- | @last_commit: $Date: 2015-03-05 10:40:28 +0100 (jue, 05 mar 2015) $
- | @version:     $Revision: 3140 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: labels.c 3140 2015-03-05 09:40:28Z harald $";
-
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -61,9 +54,8 @@ static char UNUSED rcsid[] = "$Id: labels.c 3140 2015-03-05 09:40:28Z harald $";
 #include "events.h"
 #include "labels.h"
 #include "mpi_prv_events.h"
-#include "pacx_prv_events.h"
 #include "omp_prv_events.h"
-#include "trt_prv_events.h"
+#include "java_prv_events.h"
 #include "cuda_prv_events.h"
 #include "opencl_prv_events.h"
 #include "pthread_prv_events.h"
@@ -76,7 +68,6 @@ static char UNUSED rcsid[] = "$Id: labels.c 3140 2015-03-05 09:40:28Z harald $";
 #include "object_tree.h"
 #include "utils.h"
 #include "online_events.h"
-
 #include "HardwareCounters.h"
 #include "queue.h"
 
@@ -96,7 +87,26 @@ static Extrae_Vector_t defined_basic_block_labels;
 
 static void Labels_Add_CodeLocation_Label (int eventcode, codelocation_type_t type, char *description)
 {
-	labels_codelocation = (codelocation_label_t*) realloc (labels_codelocation, (num_labels_codelocation+1)*sizeof(codelocation_label_t));
+	unsigned u;
+
+	/* Check first if this label is already included */
+	for (u = 0; u < num_labels_codelocation; u++)
+	{
+		/* If already exists, produce a warning if labels are different */
+		if (labels_codelocation[u].eventcode == eventcode &&
+		    labels_codelocation[u].type == type)
+		{
+			if (strcmp (labels_codelocation[u].description, description))
+			{
+				fprintf (stderr, PACKAGE_NAME": mpi2prv Warning! Already existing definition for event %d with a different description\n", eventcode);
+			}
+
+			return;
+		}
+	}
+
+	labels_codelocation = (codelocation_label_t*) realloc (labels_codelocation,
+	  (num_labels_codelocation+1)*sizeof(codelocation_label_t));
 	if (labels_codelocation == NULL)
 	{
 		fprintf (stderr, PACKAGE_NAME": mpi2prv Error! Cannot allocate memory to add a new code location label\n");
@@ -251,16 +261,6 @@ struct mpi_stats_evt_t mpi_stats_evt_labels[MPI_STATS_EVENTS_COUNT] = {
    { MPI_STATS_OTHER_COUNT_EV, MPI_STATS_OTHER_COUNT_LBL } 
 };
 
-struct pacx_stats_evt_t pacx_stats_evt_labels[PACX_STATS_EVENTS_COUNT] = {
-   { PACX_STATS_P2P_COMMS_EV, PACX_STATS_P2P_COMMS_LBL },
-   { PACX_STATS_P2P_BYTES_SENT_EV, PACX_STATS_P2P_BYTES_SENT_LBL },
-   { PACX_STATS_P2P_BYTES_RECV_EV, PACX_STATS_P2P_BYTES_RECV_LBL },
-   { PACX_STATS_GLOBAL_COMMS_EV, PACX_STATS_GLOBAL_COMMS_LBL },
-   { PACX_STATS_GLOBAL_BYTES_SENT_EV, PACX_STATS_GLOBAL_BYTES_SENT_LBL },
-   { PACX_STATS_GLOBAL_BYTES_RECV_EV, PACX_STATS_GLOBAL_BYTES_RECV_LBL },
-   { PACX_STATS_TIME_IN_PACX_EV, PACX_STATS_TIME_IN_PACX_LBL }
-};
-
 /******************************************************************************
  ***  state_labels
  ******************************************************************************/
@@ -568,36 +568,6 @@ static void Write_MPI_Stats_Labels (FILE * pcf_fd)
    }
 }
 
-static char * PACX_Stats_Event_Label (int pacx_stats_evt)
-{
-   int i;
-
-   for (i=0; i<MPI_STATS_EVENTS_COUNT; i++)
-   {
-      if (pacx_stats_evt_labels[i].evt_type == pacx_stats_evt) {
-         return pacx_stats_evt_labels[i].label;
-      }
-   }
-   return "Unknown PACX stats event";
-}
-
-static void Write_PACX_Stats_Labels (FILE * pcf_fd)
-{
-   int i;
-
-   if (PACX_Stats_Events_Found)
-   {
-      fprintf (pcf_fd, "%s\n", TYPE_LABEL);
-
-      for (i=0; i<PACX_STATS_EVENTS_COUNT; i++) {
-         if (PACX_Stats_Labels_Used[i]) {
-            fprintf(pcf_fd, "0    %d    %s\n", PACX_STATS_BASE+i, PACX_Stats_Event_Label(i));
-         }
-      }
-      LET_SPACES (pcf_fd);
-   }
-}
-
 static void Write_Trace_Mode_Labels (FILE * pcf_fd)
 {
 	fprintf (pcf_fd, "%s\n", TYPE_LABEL);
@@ -676,9 +646,6 @@ static void Write_Spectral_Labels (FILE * pcf_fd)
 void Labels_loadSYMfile (int taskid, int allobjects, unsigned ptask,
 	unsigned task, char *name, int report)
 {
-#ifndef HAVE_BFD
-	static int shown_BFD_missing = FALSE;
-#endif
 	static int Labels_loadSYMfile_init = FALSE;
 	FILE *FD;
 	char LINE[1024], Type;
@@ -728,16 +695,8 @@ void Labels_loadSYMfile (int taskid, int allobjects, unsigned ptask,
 						int res = sscanf (LINE, "0 \"%lx-%lx %lx %[^\n\"]\"", &start, &end, &offset, module);
 						if (res == 4)
 						{
-#ifdef HAVE_BFD
-							ObjectTable_AddBinaryObject (allobjects, ptask, task, start, end, offset, module);
-#else
-							if (!shown_BFD_missing)
-								fprintf (stdout, "mpi2prv: Ignoring symbols from the application execution because mpi2prv does not support BFD\n");
-							shown_BFD_missing = TRUE;
-							UNREFERENCED_PARAMETER(allobjects);
-							UNREFERENCED_PARAMETER(ptask);
-							UNREFERENCED_PARAMETER(task);
-#endif
+							ObjectTable_AddBinaryObject (allobjects, ptask, task,
+							  start, end, offset, module);
 						}
 						else
 							fprintf (stderr, PACKAGE_NAME": Error! Invalid line ('%s') in %s\n", LINE, name);
@@ -1030,14 +989,12 @@ int Labels_GeneratePCFfile (char *name, long long options)
 	Paraver_state_colors (fd);
 
 	MPITEvent_WriteEnabled_MPI_Operations (fd);
-	MPITEvent_WriteEnabled_PACX_Operations (fd);
 	SoftCountersEvent_WriteEnabled_MPI_Operations (fd);
-	SoftCountersEvent_WriteEnabled_PACX_Operations (fd);
 	OMPEvent_WriteEnabledOperations (fd);
 	WriteEnabled_pthread_Operations (fd);
 	MISCEvent_WriteEnabledOperations (fd, options);
-	TRTEvent_WriteEnabledOperations (fd);
 	CUDAEvent_WriteEnabledOperations (fd);
+	JavaEvent_WriteEnabledOperations (fd);
 
 #if USE_HARDWARE_COUNTERS
 	HWC_PARAVER_Labels (fd);
@@ -1055,14 +1012,14 @@ int Labels_GeneratePCFfile (char *name, long long options)
 	Address2Info_Write_OTHERS_Labels (fd, get_option_merge_UniqueCallerID(),
 		num_labels_codelocation, labels_codelocation);
 # if defined(BFD_MANAGER_GENERATE_ADDRESSES)
-	ObjectTable_dumpAddresses (fd, ADDRESSES_FOR_BINARY_EV);
+	if (get_option_dump_Addresses())
+		ObjectTable_dumpAddresses (fd, ADDRESSES_FOR_BINARY_EV);
 # endif
 #endif
 
 	Write_rusage_Labels (fd);
 	Write_memusage_Labels (fd);
 	Write_MPI_Stats_Labels (fd);
-	Write_PACX_Stats_Labels (fd);
 	Write_Trace_Mode_Labels (fd);
 	Write_Clustering_Labels (fd);
 	Write_Spectral_Labels (fd);
