@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/misc_prv_semantics.c $
- | @last_commit: $Date: 2015-04-23 17:12:21 +0200 (jue, 23 abr 2015) $
- | @version:     $Revision: 3269 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: misc_prv_semantics.c 3269 2015-04-23 15:12:21Z harald $";
-
 #include <config.h>
 
 #ifdef HAVE_STDLIB_H
@@ -80,8 +73,6 @@ int Memusage_Events_Found = FALSE;
 int Memusage_Labels_Used[MEMUSAGE_EVENTS_COUNT];
 int MPI_Stats_Events_Found = FALSE;
 int MPI_Stats_Labels_Used[MPI_STATS_EVENTS_COUNT];
-int PACX_Stats_Events_Found = FALSE;
-int PACX_Stats_Labels_Used[PACX_STATS_EVENTS_COUNT];
 
 unsigned int MaxClusterId = 0; /* Marks the maximum cluster id assigned in the mpits */
 
@@ -115,7 +106,7 @@ static int Flush_Event (event_t * current_event,
 }
 
 /******************************************************************************
- ***  Read_Event
+ ***  ReadWrite_Event
  ******************************************************************************/
 
 static int ReadWrite_Event (event_t * event, unsigned long long time,
@@ -143,13 +134,17 @@ static int ReadWrite_Event (event_t * event, unsigned long long time,
 			case EVT_BEGIN:
 			trace_paraver_event (cpu, ptask, task, thread, time, IO_EV,
 		  	  EvType==READ_EV?READ_VAL_EV:WRITE_VAL_EV);
-			trace_paraver_event (cpu, ptask, task, thread, time, IO_DESCRIPTOR_EV,
-			  EvParam);
+			trace_paraver_event (cpu, ptask, task, thread, time,
+			  IO_DESCRIPTOR_EV, EvParam);
 			break;
 			case EVT_BEGIN+1:
 			trace_paraver_event (cpu, ptask, task, thread, time, IO_SIZE_EV,
 			  EvParam);
 			break;
+			case EVT_BEGIN+2:
+			trace_paraver_event (cpu, ptask, task, thread, time,
+			  IO_DESCRIPTOR_TYPE_EV, EvParam);
+			break;
 			default:
 			break;
 		}
@@ -218,7 +213,10 @@ static int Appl_Event (event_t * current_event,
 		Pop_State (STATE_ANY, ptask, task, thread);
 
 	trace_paraver_state (cpu, ptask, task, thread, current_time);
-	trace_paraver_event (cpu, ptask, task, thread, current_time, EvType, EvValue);
+	trace_paraver_event (cpu, ptask, task, thread, current_time, EvType,
+	  EvValue);
+	trace_paraver_event (cpu, ptask, task, thread, current_time,
+	  CLOCK_FROM_SYSTEM_EV, Get_EvTime (current_event));
 
 	return 0;
 }
@@ -300,14 +298,15 @@ static int MPI_Caller_Event (event_t * current_event,
                              unsigned int thread,
                              FileSet_t *fset)
 {
-	int i, deepness;	
+	thread_t *thread_info = GET_THREAD_INFO(ptask, task, thread);
+	unsigned i, deepness;	
 	UINT64 EvValue = Get_EvValue(current_event);
 	UNREFERENCED_PARAMETER(fset);
 
 	trace_paraver_state (cpu, ptask, task, thread, current_time);
 
 	deepness = Get_EvEvent(current_event) - CALLER_EV;
-	if (deepness > 0) 
+	if (deepness > 0 && deepness < MAX_CALLERS) 
 	{
 		MPI_Caller_Multiple_Levels_Traced = TRUE;	
 		if (MPI_Caller_Labels_Used == NULL) 
@@ -327,13 +326,20 @@ static int MPI_Caller_Event (event_t * current_event,
 #if defined(HAVE_BFD)
 	if (get_option_merge_SortAddresses())
 	{
-		AddressCollector_Add (&CollectedAddresses, ptask, task, EvValue, ADDR2MPI_FUNCTION);
-		AddressCollector_Add (&CollectedAddresses, ptask, task, EvValue, ADDR2MPI_LINE);
+		AddressCollector_Add (&CollectedAddresses, ptask, task, EvValue,
+		  ADDR2MPI_FUNCTION);
+		AddressCollector_Add (&CollectedAddresses, ptask, task, EvValue,
+		  ADDR2MPI_LINE);
 	}
 #endif
 
-	trace_paraver_event (cpu, ptask, task, thread, current_time, CALLER_EV+deepness, EvValue);
-	trace_paraver_event (cpu, ptask, task, thread, current_time, CALLER_LINE_EV+deepness, EvValue);
+	trace_paraver_event (cpu, ptask, task, thread, current_time,
+	  CALLER_EV+deepness, EvValue);
+	trace_paraver_event (cpu, ptask, task, thread, current_time,
+	  CALLER_LINE_EV+deepness, EvValue);
+
+	if (deepness > 0 && deepness < MAX_CALLERS)
+		thread_info->AddressSpace_calleraddresses[deepness] = EvValue;
 
 	return 0;
 }
@@ -449,42 +455,6 @@ static int MPI_Stats_Event (
 }
 
 
-/******************************************************************************
- ***  PACX_Stats_Event
- ******************************************************************************/
-static int PACX_Stats_Event (
-   event_t * current_event,
-   unsigned long long current_time,
-   unsigned int cpu,
-   unsigned int ptask,
-   unsigned int task,
-   unsigned int thread,
-   FileSet_t *fset )
-{
-	int i;
-	unsigned int EvType;
-	unsigned long long EvValue;
-	UNREFERENCED_PARAMETER(fset);
-
-	EvType  = Get_EvValue (current_event);     /* Value is the event type.  */
-	EvValue = Get_EvMiscParam (current_event); /* Param is the event value. */
-
-	trace_paraver_state (cpu, ptask, task, thread, current_time);
-	trace_paraver_event (cpu, ptask, task, thread, current_time, PACX_STATS_BASE+EvType, EvValue);
-
-	if (!PACX_Stats_Events_Found)
-	{
-		PACX_Stats_Events_Found = TRUE;
-		for (i=0; i<PACX_STATS_EVENTS_COUNT; i++)
-		{
-			PACX_Stats_Labels_Used[i] = FALSE;
-		}
-	}
-	PACX_Stats_Labels_Used[EvType] = TRUE;
-
-	return 0;
-}
-
 /******************************************************************************
  ***  InitTracing_Event
  ******************************************************************************/
@@ -542,8 +512,7 @@ static int USRFunction_Event (event_t * current,
 
 	trace_paraver_state (cpu, ptask, task, thread, current_time);
 	trace_paraver_event (cpu, ptask, task, thread, current_time, USRFUNC_EV, EvValue);
-	if (EvValue != 0)
-		trace_paraver_event (cpu, ptask, task, thread, current_time, USRFUNC_LINE_EV, EvValue);
+	trace_paraver_event (cpu, ptask, task, thread, current_time, USRFUNC_LINE_EV, EvValue);
 
 	return 0;
 }
@@ -555,11 +524,15 @@ static int Sampling_Address_Event (event_t * current,
 	unsigned long long current_time, unsigned int cpu, unsigned int ptask,
 	unsigned int task, unsigned int thread, FileSet_t *fset)
 {
+	uint64_t *CallerAddresses;
 	unsigned i;
+	int EvType;
 	UINT64 EvValue;
 	UINT64 EvParam;
+	task_t *task_info = GET_TASK_INFO(ptask, task);
 	UNREFERENCED_PARAMETER(fset);
 
+	EvType = Get_EvEvent (current);
 	EvValue = Get_EvValue (current);
 	EvParam = Get_EvMiscParam (current); /* Param is the sampled address */
 
@@ -591,7 +564,27 @@ static int Sampling_Address_Event (event_t * current,
 	}
 
 	if (EvParam != 0)
-		trace_paraver_event (cpu, ptask, task, thread, current_time, SAMPLING_ADDRESS_EV, EvParam);
+		trace_paraver_event (cpu, ptask, task, thread, current_time, EvType, EvParam);
+
+	if (AddressSpace_search (task_info->AddressSpace, EvParam, &CallerAddresses,
+	    NULL))
+	{
+		unsigned u;
+		for (u = 0; u < MAX_CALLERS; u++)
+		{
+			if (CallerAddresses[u] != 0)
+			{
+				trace_paraver_event (cpu, ptask, task, thread, current_time,
+				  SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV+u,
+				  CallerAddresses[u]);
+			}
+		}
+		trace_paraver_event (cpu, ptask, task, thread, current_time,
+		  SAMPLING_ADDRESS_ALLOCATED_OBJECT_EV, 0);
+	}
+	else
+		trace_paraver_event (cpu, ptask, task, thread, current_time,
+		  SAMPLING_ADDRESS_STATIC_OBJECT_EV, EvParam);
 
 	return 0;
 }
@@ -1339,7 +1332,8 @@ static int Register_Stacked_Type_Event (event_t * current_event,
 	UNREFERENCED_PARAMETER(thread);
 	UNREFERENCED_PARAMETER(fset);
 
-	Vector_Add (RegisteredStackValues, Get_EvValue(current_event));
+	if (!Vector_Search (RegisteredStackValues, Get_EvValue(current_event)))
+		Vector_Add (RegisteredStackValues, Get_EvValue(current_event));
 
 	return 0;
 }
@@ -1370,7 +1364,11 @@ static int Register_CodeLocation_Type_Event (event_t * current_event,
 	cl_types = Extrae_Addr2Type_New (EvFunction, ADDR2OTHERS_FUNCTION,
 		EvLine, ADDR2OTHERS_LINE);
 
-	Extrae_Vector_Append (&RegisteredCodeLocationTypes, cl_types);
+	if (!Extrae_Vector_Search (&RegisteredCodeLocationTypes, cl_types,
+	     Extrae_Addr2Type_Compare))
+	{
+		Extrae_Vector_Append (&RegisteredCodeLocationTypes, cl_types);
+	}
 
 	return 0;
 }
@@ -1461,6 +1459,8 @@ static int DynamicMemory_Event (event_t * event,
 	unsigned long long time, unsigned int cpu, unsigned int ptask,
 	unsigned int task, unsigned int thread, FileSet_t *fset)
 {
+	task_t *task_info = GET_TASK_INFO(ptask, task);
+	thread_t *thread_info = GET_THREAD_INFO(ptask, task, thread);
 	UNREFERENCED_PARAMETER(fset);
 
 	unsigned EvType = Get_EvEvent (event);
@@ -1468,55 +1468,125 @@ static int DynamicMemory_Event (event_t * event,
 	unsigned long long EvValue = Get_EvValue (event);
 	int isBegin = EvValue == EVT_BEGIN;
 
-	if (EvValue == EVT_BEGIN || EvValue == EVT_END)
-	{
-		unsigned PRVValue = isBegin?MISC_event_GetValueForDynamicMemory(EvType):0;
-		Switch_State (STATE_OTHERS, EvValue == EVT_BEGIN, ptask, task, thread);
-		trace_paraver_state (cpu, ptask, task, thread, time);
-		trace_paraver_event (cpu, ptask, task, thread, time, DYNAMIC_MEM_EV, PRVValue);
-	}
-
 	if (EvType == MALLOC_EV)
 	{
 		/* Malloc: in size, out pointer */
 		if (isBegin)
+		{
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_REQUESTED_SIZE_EV, EvParam);
+
+			/* Store size and time of creation for later use in isEnd */
+			thread_info->AddressSpace_size = EvParam;
+			thread_info->AddressSpace_timeCreation = time;
+		}
 		else
+		{
+			/* Emit information regarding the calling site to identify
+			   where the structure was allocated */
+			unsigned u;
+			for (u = 0; u < MAX_CALLERS; u++)
+			{
+				if (thread_info->AddressSpace_calleraddresses[u] != 0)
+					trace_paraver_event (cpu, ptask, task, thread,
+					  thread_info->AddressSpace_timeCreation,
+					  SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV+u,
+					  thread_info->AddressSpace_calleraddresses[u]);
+			}
+			/* Emit event that will be replaced by the ID of the 
+			   location of structure allocation */
+			trace_paraver_event (cpu, ptask, task, thread,
+			  thread_info->AddressSpace_timeCreation,
+			  SAMPLING_ADDRESS_ALLOCATED_OBJECT_EV, 0);
+
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_POINTER_OUT_EV, EvParam);
+
+			AddressSpace_add (task_info->AddressSpace, EvParam,
+			  EvParam+thread_info->AddressSpace_size,
+			  thread_info->AddressSpace_calleraddresses,
+			  thread_info->AddressSpace_callertype);
+		}
 	}
 	else if (EvType == FREE_EV)
 	{
 		/* Free: in pointer */
 		if (isBegin)
+		{
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_POINTER_IN_EV, EvParam);
+
+			AddressSpace_remove (task_info->AddressSpace, EvParam);
+		}
 	}
 	else if (EvType == REALLOC_EV)
 	{
 		/* Realloc: in size, in pointer (in EVT_BEGIN+1), out ptr*/
 		if (EvValue == EVT_BEGIN)
+		{
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_POINTER_IN_EV, EvParam);
+
+			thread_info->AddressSpace_size = EvParam;
+		}
 		else if (EvValue == EVT_BEGIN+1)
+		{
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_REQUESTED_SIZE_EV, EvParam);
+
+			AddressSpace_remove (task_info->AddressSpace, EvParam);
+		}
 		else
+		{
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_POINTER_OUT_EV, EvParam);
+
+			AddressSpace_add (task_info->AddressSpace, EvParam,
+			  EvParam+thread_info->AddressSpace_size,
+			  thread_info->AddressSpace_calleraddresses,
+			  thread_info->AddressSpace_callertype);
+		}
 	
 	}
 	else if (EvType == CALLOC_EV)
 	{
 		/* Calloc: in size, out pointer */
 		if (isBegin)
+		{
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_REQUESTED_SIZE_EV, EvParam);
+
+			thread_info->AddressSpace_size = EvParam;
+		}
 		else
+		{
 			trace_paraver_event (cpu, ptask, task, thread, time,
 			  DYNAMIC_MEM_POINTER_OUT_EV, EvParam);
+
+			AddressSpace_add (task_info->AddressSpace, EvParam,
+			  EvParam+thread_info->AddressSpace_size,
+			  thread_info->AddressSpace_calleraddresses,
+			  thread_info->AddressSpace_callertype);
+		}
 	}
+
+	if (EvValue == EVT_BEGIN || EvValue == EVT_END)
+	{
+		// Do not change the state in MALLOC related calls
+		// Switch_State (STATE_OTHERS, EvValue == EVT_BEGIN, ptask, task, thread);
+		// trace_paraver_state (cpu, ptask, task, thread, time);
+
+		unsigned PRVValue = isBegin?MISC_event_GetValueForDynamicMemory(EvType):0;
+		trace_paraver_event (cpu, ptask, task, thread, time, DYNAMIC_MEM_EV, PRVValue);
+	}
+
+	if (!isBegin)
+	{
+		unsigned u;
+		for (u = 0; u < MAX_CALLERS; u++)
+			thread_info->AddressSpace_calleraddresses[u] = 0;
+	}
+
 	return 0;
 }
 
@@ -1545,7 +1615,6 @@ SingleEv_Handler_t PRV_MISC_Event_Handlers[] = {
 	{ RUSAGE_EV, GetRusage_Event },
 	{ MEMUSAGE_EV, Memusage_Event },
 	{ MPI_STATS_EV, MPI_Stats_Event },
-	{ PACX_STATS_EV, PACX_Stats_Event },
 	{ USRFUNC_EV, USRFunction_Event },
 	{ TRACING_MODE_EV, Tracing_Mode_Event },
 	{ ONLINE_EV, Online_Event },
@@ -1561,7 +1630,8 @@ SingleEv_Handler_t PRV_MISC_Event_Handlers[] = {
 	{ WAITPID_EV, ForkWaitSystem_Event },
 	{ EXEC_EV, Exec_Event },
 	{ GETCPU_EV, GetCPU_Event },
-	{ SAMPLING_ADDRESS_EV, Sampling_Address_Event },
+	{ SAMPLING_ADDRESS_LD_EV, Sampling_Address_Event },
+	{ SAMPLING_ADDRESS_ST_EV, Sampling_Address_Event },
 	{ SAMPLING_ADDRESS_MEM_LEVEL_EV, Sampling_Address_MEM_TLB_Event },
 	{ SAMPLING_ADDRESS_TLB_LEVEL_EV, Sampling_Address_MEM_TLB_Event },
 	{ SAMPLING_ADDRESS_REFERENCE_COST_EV, Sampling_Address_MEM_TLB_Event },
