@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/mpi_prv_events.c $
- | @last_commit: $Date: 2015-02-23 12:32:16 +0100 (lun, 23 feb 2015) $
- | @version:     $Revision: 3120 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: mpi_prv_events.c 3120 2015-02-23 11:32:16Z harald $";
-
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -59,7 +52,7 @@ struct t_prv_val_label
   char *label;
 };
 
-#define NUM_MPI_PRV_ELEMENTS 153    /* 127 */
+#define NUM_MPI_PRV_ELEMENTS 167
 
 static struct t_event_mpit2prv event_mpit2prv[NUM_MPI_PRV_ELEMENTS] = {
 	{MPI_ALLGATHER_EV, MPITYPE_COLLECTIVE, MPI_ALLGATHER_VAL, FALSE}, /*   1 */
@@ -212,13 +205,26 @@ static struct t_event_mpit2prv event_mpit2prv[NUM_MPI_PRV_ELEMENTS] = {
 	{MPI_FILE_READ_AT_ALL_EV, MPITYPE_IO, MPI_FILE_READ_AT_ALL_VAL, FALSE}, /* 148 */
 	{MPI_FILE_WRITE_AT_EV, MPITYPE_IO, MPI_FILE_WRITE_AT_VAL, FALSE}, /* 149 */
 	{MPI_FILE_WRITE_AT_ALL_EV, MPITYPE_IO, MPI_FILE_WRITE_AT_ALL_VAL, FALSE}, /* 150 */
-	{MPI_COMM_SPAWN_EV, MPITYPE_COMM, MPI_COMM_SPAWN_VAL, FALSE}, /* 151 */
-	{MPI_COMM_SPAWN_MULTIPLE_EV, MPITYPE_COMM, MPI_COMM_SPAWN_MULTIPLE_VAL, FALSE}, /* 152 */
-	{MPI_REQUEST_GET_STATUS_EV, MPITYPE_OTHER, MPI_REQUEST_GET_STATUS_VAL, FALSE}        /*  153 */
+	{MPI_COMM_SPAWN_EV, MPITYPE_COMM, MPI_COMM_SPAWN_VAL, FALSE},
+	{MPI_COMM_SPAWN_MULTIPLE_EV, MPITYPE_COMM, MPI_COMM_SPAWN_MULTIPLE_VAL, FALSE},
+	{MPI_REQUEST_GET_STATUS_EV, MPITYPE_OTHER, MPI_REQUEST_GET_STATUS_VAL, FALSE},
+	{MPI_IREDUCE_EV, MPITYPE_COLLECTIVE, MPI_IREDUCE_VAL, FALSE},
+	{MPI_IALLREDUCE_EV, MPITYPE_COLLECTIVE, MPI_IALLREDUCE_VAL, FALSE}, /* 155 */
+	{MPI_IBARRIER_EV, MPITYPE_COLLECTIVE, MPI_IBARRIER_VAL, FALSE},
+	{MPI_IBCAST_EV, MPITYPE_COLLECTIVE, MPI_IBCAST_VAL, FALSE},
+	{MPI_IALLTOALL_EV, MPITYPE_COLLECTIVE, MPI_ALLTOALL_VAL, FALSE},
+	{MPI_IALLTOALLV_EV, MPITYPE_COLLECTIVE, MPI_IALLTOALLV_VAL, FALSE},
+	{MPI_IALLGATHER_EV, MPITYPE_COLLECTIVE, MPI_IALLGATHER_VAL, FALSE}, /* 160 */
+	{MPI_IALLGATHERV_EV, MPITYPE_COLLECTIVE, MPI_IALLGATHERV_VAL, FALSE},
+	{MPI_IGATHER_EV, MPITYPE_COLLECTIVE, MPI_IGATHER_VAL, FALSE},
+	{MPI_IGATHERV_EV, MPITYPE_COLLECTIVE, MPI_IGATHERV_VAL, FALSE},
+	{MPI_ISCATTER_EV, MPITYPE_COLLECTIVE, MPI_ISCATTER_VAL, FALSE},
+	{MPI_ISCATTERV_EV, MPITYPE_COLLECTIVE, MPI_ISCATTERV_VAL, FALSE}, /* 165 */
+	{MPI_IREDUCESCAT_EV, MPITYPE_COLLECTIVE, MPI_IREDUCESCAT_VAL, FALSE},
+	{MPI_ISCAN_EV, MPITYPE_COLLECTIVE, MPI_ISCAN_VAL, FALSE}
 };
 
 
-
 /* Dels 12, de moment nomes 8 son diferents */
 static struct t_prv_type_info prv_block_groups[NUM_MPI_BLOCK_GROUPS] = {
 	{MPITYPE_PTOP, MPITYPE_PTOP_LABEL, MPITYPE_FLAG_COLOR},
@@ -392,7 +398,21 @@ static struct t_prv_val_label mpi_prv_val_label[NUM_MPI_PRV_ELEMENTS] = {
 	{MPI_FILE_READ_AT_VAL, MPI_FILE_READ_AT_LABEL},
 	{MPI_FILE_READ_AT_ALL_VAL, MPI_FILE_READ_AT_ALL_LABEL},
 	{MPI_FILE_WRITE_AT_VAL, MPI_FILE_WRITE_AT_LABEL},
-	{MPI_FILE_WRITE_AT_ALL_VAL, MPI_FILE_WRITE_AT_ALL_LABEL}
+	{MPI_FILE_WRITE_AT_ALL_VAL, MPI_FILE_WRITE_AT_ALL_LABEL},
+	{MPI_IREDUCE_VAL, MPI_IREDUCE_LABEL},
+	{MPI_IALLREDUCE_VAL, MPI_IALLREDUCE_LABEL},
+	{MPI_IBARRIER_VAL, MPI_IBARRIER_LABEL},
+	{MPI_IBCAST_VAL, MPI_IBCAST_LABEL},
+	{MPI_IALLTOALL_VAL, MPI_IALLTOALL_LABEL},
+	{MPI_ALLTOALLV_VAL, MPI_IALLTOALLV_LABEL},
+	{MPI_IALLGATHER_VAL, MPI_IALLGATHER_LABEL},
+	{MPI_IALLGATHER_VAL, MPI_IALLGATHERV_LABEL},
+	{MPI_IGATHER_VAL, MPI_IGATHER_LABEL},
+	{MPI_IGATHERV_VAL, MPI_IGATHERV_LABEL},
+	{MPI_ISCATTER_VAL, MPI_ISCATTER_LABEL},
+	{MPI_ISCATTERV_VAL, MPI_ISCATTERV_LABEL},
+	{MPI_IREDUCESCAT_VAL, MPI_IREDUCESCAT_LABEL},
+	{MPI_ISCAN_VAL, MPI_ISCAN_LABEL}
 };
 
 
@@ -402,10 +422,11 @@ static struct t_prv_val_label mpi_prv_val_label[NUM_MPI_PRV_ELEMENTS] = {
 #define COLLECTIVE_INDEX                                    3
 #define MPI_REQUEST_GET_STATUS_CNT_INDEX                    4
 #define TIME_OUTSIDE_MPI_REQUEST_GET_STATUS_INDEX           5
+#define IO_INDEX                                            6
 
-#define MAX_SOFTCNT                                         6
+#define MAX_SOFTCNT                                         7
 
-int MPI_SoftCounters_used[MAX_SOFTCNT] = { FALSE, FALSE, FALSE, FALSE, FALSE, FALSE };
+int MPI_SoftCounters_used[MAX_SOFTCNT] = { FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE };
 
 void Enable_MPI_Soft_Counter (unsigned int EvType)
 {
@@ -425,8 +446,24 @@ void Enable_MPI_Soft_Counter (unsigned int EvType)
 	         EvType == MPI_ALLGATHER_EV || EvType == MPI_ALLGATHERV_EV ||
 	         EvType == MPI_GATHER_EV || EvType == MPI_GATHERV_EV ||
 	         EvType == MPI_SCATTER_EV || EvType == MPI_SCATTERV_EV ||
-	         EvType == MPI_REDUCESCAT_EV || EvType == MPI_SCAN_EV)
+	         EvType == MPI_REDUCESCAT_EV || EvType == MPI_SCAN_EV ||
+	         EvType == MPI_IREDUCE_EV || EvType == MPI_IALLREDUCE_EV ||
+	         EvType == MPI_IBARRIER_EV || EvType == MPI_IBCAST_EV ||
+	         EvType == MPI_IALLTOALL_EV || EvType == MPI_IALLTOALLV_EV ||
+	         EvType == MPI_IALLGATHER_EV || EvType == MPI_IALLGATHERV_EV ||
+	         EvType == MPI_IGATHER_EV || EvType == MPI_IGATHERV_EV ||
+	         EvType == MPI_ISCATTER_EV || EvType == MPI_ISCATTERV_EV ||
+	         EvType == MPI_IREDUCESCAT_EV || EvType == MPI_ISCAN_EV)
 		MPI_SoftCounters_used[COLLECTIVE_INDEX] = TRUE;
+        else if (EvType == MPI_FILE_READ_EV        ||
+                 EvType == MPI_FILE_READ_ALL_EV    ||
+                 EvType == MPI_FILE_WRITE_EV       ||
+                 EvType == MPI_FILE_WRITE_ALL_EV   ||
+                 EvType == MPI_FILE_READ_AT_EV     ||
+                 EvType == MPI_FILE_READ_AT_ALL_EV ||
+                 EvType == MPI_FILE_WRITE_AT_EV    ||
+                 EvType == MPI_FILE_WRITE_AT_ALL_EV)
+		MPI_SoftCounters_used[IO_INDEX] = TRUE;
 }
 
 
@@ -585,6 +622,7 @@ void MPITEvent_WriteEnabled_MPI_Operations (FILE * fd)
 #define TEST_COUNTER_LBL                            "MPI_Test misses"
 #define MPI_REQUEST_GET_STATUS_COUNTER_LBL          "MPI_Request_get_status counter"
 #define TIME_OUTSIDE_MPI_REQUEST_GET_STATUS_LBL     "Elapsed time outside MPI_Request_get_status"
+#define IO_SIZE_LBL                                 "MPI-IO size in bytes"
 
 void SoftCountersEvent_WriteEnabled_MPI_Operations (FILE * fd)
 {
@@ -636,7 +674,13 @@ void SoftCountersEvent_WriteEnabled_MPI_Operations (FILE * fd)
 			MPI_TIME_OUTSIDE_MPI_REQUEST_GET_STATUS_EV, TIME_OUTSIDE_MPI_REQUEST_GET_STATUS_LBL);
 		LET_SPACES(fd);
 	}
-
+	if (MPI_SoftCounters_used[IO_INDEX])
+	{
+                fprintf (fd, "EVENT_TYPE\n");
+                fprintf (fd, "%d    %d    %s\n\n", 0,
+                        MPI_IO_SIZE_EV, IO_SIZE_LBL);
+                LET_SPACES(fd);
+	}
 }
 
 void Translate_MPI_MPIT2PRV (int typempit, UINT64 valuempit, int *typeprv, UINT64 *valueprv)
