@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/paraver_generator.c $
- | @last_commit: $Date: 2015-03-10 17:22:45 +0100 (mar, 10 mar 2015) $
- | @version:     $Revision: 3162 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: paraver_generator.c 3162 2015-03-10 16:22:45Z harald $";
-
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -89,7 +82,6 @@ static char UNUSED rcsid[] = "$Id: paraver_generator.c 3162 2015-03-10 16:22:45Z
 #include "options.h"
 
 #include "mpi_prv_events.h"
-#include "pacx_prv_events.h"
 #include "addr2info.h"
 
 #if USE_HARDWARE_COUNTERS
@@ -136,6 +128,52 @@ static void trace_paraver_recordAt (WriteFileBuffer_t *wfb,
 	WriteFileBuffer_writeAt (wfb, record, position);
 }
 
+/******************************************************************************
+ ***  trace_paraver_state_noahead
+ ******************************************************************************/
+void trace_paraver_state_noahead (
+   unsigned int cpu, unsigned int ptask, unsigned int task, unsigned int thread,
+   unsigned long long current_time)
+{
+	thread_t * thread_info = GET_THREAD_INFO (ptask, task, thread);
+	WriteFileBuffer_t *wfb = thread_info->file->wfb;
+	unsigned current_state = Top_State(ptask, task, thread);
+
+	UNREFERENCED_PARAMETER(cpu);
+
+#if 0
+	fprintf (stderr, "trace_paraver_state (..)\n");
+	fprintf (stderr, "thread_info->incomplete_state_offset = %u\n", thread_info->incomplete_state_offset);
+#endif
+
+	/* Complete the previous state */
+	if (thread_info->incomplete_state_offset != (off_t)-1) /* This isn't the first state */
+	{
+#if 0
+		fprintf (stderr, "get_option_merge_JointStates() = %d Get_Last_State() = %d\n", get_option_merge_JointStates(), Get_Last_State());
+		fprintf (stderr, "thread_info->incomplete_state_record.value = %d == current_state = %d\n", thread_info->incomplete_state_record.value, current_state);
+#endif
+
+		/* Do not split states whether appropriate */
+		if (get_option_merge_JointStates() && !Get_Last_State())
+			if (thread_info->incomplete_state_record.value == current_state)
+				return;
+
+		/* Write the record into the *.tmp file if the state isn't excluded */
+#if defined(DEBUG_STATES)
+		fprintf(stderr, "mpi2prv: DEBUG [T:%d] Closing state %u at %llu\n", task,  
+		(unsigned int)thread_info->incomplete_state_record.value, current_time);
+		fprintf (stderr, "Excluded? %d\n", State_Excluded(thread_info->incomplete_state_record.value));
+#endif
+
+		if (!State_Excluded(thread_info->incomplete_state_record.value))
+		{
+			thread_info->incomplete_state_record.end_time = current_time;
+			WriteFileBuffer_writeAt (wfb, &(thread_info->incomplete_state_record), thread_info->incomplete_state_offset);
+		}
+	}
+}
+
 /******************************************************************************
  ***  trace_paraver_state
  ******************************************************************************/
@@ -143,10 +181,9 @@ void trace_paraver_state (
    unsigned int cpu, unsigned int ptask, unsigned int task, unsigned int thread,
    unsigned long long current_time)
 {
-	unsigned int current_state;
 	thread_t * thread_info = GET_THREAD_INFO (ptask, task, thread);
 	WriteFileBuffer_t *wfb = thread_info->file->wfb;
-	current_state = Top_State(ptask, task, thread);
+	unsigned current_state = Top_State(ptask, task, thread);
 
 #if 0
 	fprintf (stderr, "trace_paraver_state (..)\n");
@@ -195,7 +232,11 @@ void trace_paraver_state (
 	if (!State_Excluded(current_state))
 	{
 		paraver_rec_t fake_record;
-		fake_record.type = UNFINISHED_STATE;
+		fake_record.type   = UNFINISHED_STATE;
+		fake_record.ptask  = ptask;
+		fake_record.task   = task;
+		fake_record.thread = thread;
+		fake_record.time   = current_time;
 		thread_info->incomplete_state_offset = WriteFileBuffer_getPosition (wfb);
 		trace_paraver_record (wfb, &fake_record);
 	}
@@ -226,10 +267,6 @@ void trace_paraver_event (
 	{
 		Translate_MPI_MPIT2PRV (type, value, &tipus, &valor);
 	}
-	else if (type >= PACX_MIN_EV && type <= PACX_MAX_EV)
-	{
-		Translate_PACX_MPIT2PRV (type, value, &tipus, &valor);
-	}
 	else
 	{
 		tipus = type;
@@ -372,24 +409,13 @@ int trace_paraver_pending_communication (unsigned int cpu_s,
 void trace_enter_global_op (unsigned int cpu, unsigned int ptask,
 	unsigned int task, unsigned int thread, unsigned long long time,
 	unsigned int com_id, unsigned int send_size, unsigned int recv_size,
-	unsigned int is_root, unsigned isMPI)
+	unsigned int is_root)
 {
-	if (isMPI)
-	{
-		trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_SENDSIZE, send_size);
-		trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_RECVSIZE, recv_size);
-		trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_COMM, com_id);
-		if (is_root)
-			trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_ROOT, is_root);
-	}
-	else
-	{
-		trace_paraver_event (cpu, ptask, task, thread, time, PACX_GLOBAL_OP_SENDSIZE, send_size);
-		trace_paraver_event (cpu, ptask, task, thread, time, PACX_GLOBAL_OP_RECVSIZE, recv_size);
-		trace_paraver_event (cpu, ptask, task, thread, time, PACX_GLOBAL_OP_COMM, com_id);
-		if (is_root)
-			trace_paraver_event (cpu, ptask, task, thread, time, PACX_GLOBAL_OP_ROOT, is_root);
-	}
+	trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_SENDSIZE, send_size);
+	trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_RECVSIZE, recv_size);
+	trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_COMM, com_id);
+	if (is_root)
+		trace_paraver_event (cpu, ptask, task, thread, time, MPI_GLOBAL_OP_ROOT, is_root);
 }
 
 #if defined(NEW_PRINTF)
@@ -649,7 +675,12 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 	int prev_cpu, prev_ptask, prev_task, prev_thread;
 	unsigned long long prev_time;
 	paraver_rec_t *cur;
-	int i = 0;
+	UINT64 CallerAddresses[MAX_CALLERS];
+	unsigned nevents = 0;
+
+	// Here we store the caller addresses for a reference to a dynamic mem object
+	// Set to 0 initially
+	memset (CallerAddresses, 0, sizeof(CallerAddresses));
 
 	cur = *current;
 
@@ -665,14 +696,19 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 		if (prev_cpu == cur->cpu && prev_ptask == cur->ptask &&
 		prev_task == cur->task && prev_thread == cur->thread &&
 		prev_time == cur->time && cur->type == 2 &&
-		i < MAX_EVENT_COUNT_IN_MULTI_EVENT)
+		nevents < MAX_EVENT_COUNT_IN_MULTI_EVENT)
 		{
 			/* Copy the value by default... we'll change it if needed */
-			values[i] = cur->value;
-			events[i] = cur->event;
+			values[nevents] = cur->value;
+			events[nevents] = cur->event;
+
+#if defined(DEBUG)
+			fprintf (stderr, "mpi2prv: paraver_build_multi_event %d:%d:%d <%d,%llu> @ %llu\n",
+			  prev_ptask, prev_task, prev_thread, events[nevents], values[nevents], prev_time);
+#endif
 
 			if (cur->event == MPI_GLOBAL_OP_COMM)
-				values[i] = (UINT64)alies_comunicador ((int) cur->value, cur->ptask, cur->task);
+				values[nevents] = (UINT64)alies_comunicador ((int) cur->value, cur->ptask, cur->task);
 #if defined(HAVE_BFD)
 			else
 			{
@@ -687,7 +723,34 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 				  cur->event == PTHREAD_FUNC_EV || cur->event == PTHREAD_FUNC_LINE_EV ||
 				  cur->event == CUDAFUNC_EV || cur->event == CUDAFUNC_LINE_EV)
 				{
-					values[i] = paraver_translate_bfd_event (cur->ptask, cur->task, cur->event, cur->value);
+					values[nevents] = paraver_translate_bfd_event (cur->ptask,
+					  cur->task, cur->event, cur->value);
+				}
+
+				if (cur->event >= SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV &&
+				    cur->event < SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV+MAX_CALLERS)
+				{
+					CallerAddresses[cur->event-SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV] =
+					  cur->value;
+				}
+
+				if (cur->event == SAMPLING_ADDRESS_ALLOCATED_OBJECT_EV)
+				{
+					values[nevents] = Address2Info_Translate_MemReference (cur->ptask,
+					  cur->task, cur->value, MEM_REFERENCE_DYNAMIC,
+					  CallerAddresses);
+
+					// Set to 0 again after emitting the information
+					memset (CallerAddresses, 0, sizeof(CallerAddresses));
+				}
+				else if (cur->event == SAMPLING_ADDRESS_STATIC_OBJECT_EV)
+				{
+					values[nevents] = Address2Info_Translate_MemReference (cur->ptask,
+					  cur->task, cur->value, MEM_REFERENCE_STATIC, NULL);
+					events[nevents]  = SAMPLING_ADDRESS_ALLOCATED_OBJECT_EV;
+
+					// Set to 0 again after emitting the information
+					memset (CallerAddresses, 0, sizeof(CallerAddresses));
 				}
 
 				if (Extrae_Vector_Count (&RegisteredCodeLocationTypes) > 0)
@@ -699,8 +762,10 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 						Extrae_Addr2Type_t *element = 
 							Extrae_Vector_Get (&RegisteredCodeLocationTypes, u);
 
-						if (element->FunctionType == cur->event || element->LineType == cur->event)
-							values[i] = paraver_translate_bfd_event (cur->ptask, cur->task, cur->event, cur->value);
+						if (element->FunctionType == cur->event ||
+						    element->LineType == cur->event)
+							values[nevents] = paraver_translate_bfd_event (cur->ptask,
+							  cur->task, cur->event, cur->value);
 					}
 				}
 
@@ -712,11 +777,11 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 					  cur->event == OMPFUNC_EV || cur->event == TASKFUNC_INST_EV ||
 					  cur->event == PTHREAD_FUNC_EV || cur->event == CUDAFUNC_EV)
 					{
-						if (values[i] == UNRESOLVED_ID+1 || values[i] == NOT_FOUND_ID+1)
+						if (cur->value == UNRESOLVED_ID+1 || cur->value == NOT_FOUND_ID+1)
 						{
-							i++;
-							events[i] = LIBRARY_EV;
-							values[i] = Address2Info_GetLibraryID (cur->ptask, cur->task, cur->value);
+							nevents++;
+							events[nevents] = LIBRARY_EV;
+							values[nevents] = Address2Info_GetLibraryID (cur->ptask, cur->task, cur->value);
 						}
 					}
 					else
@@ -731,11 +796,11 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 									Extrae_Vector_Get (&RegisteredCodeLocationTypes, u);
 	
 								if (element->FunctionType == cur->event || element->LineType == cur->event)
-									if (values[i] == UNRESOLVED_ID+1 || values[i] == NOT_FOUND_ID+1)
+									if (cur->value == UNRESOLVED_ID+1 || cur->value == NOT_FOUND_ID+1)
 									{
-										i++;
-										events[i] = LIBRARY_EV;
-										values[i] = Address2Info_GetLibraryID (cur->ptask, cur->task, cur->value);
+										nevents++;
+										events[nevents] = LIBRARY_EV;
+										values[nevents] = Address2Info_GetLibraryID (cur->ptask, cur->task, cur->value);
 									}
 							}
 						}
@@ -743,7 +808,11 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 				}
 			}
 #endif
-			i++;
+
+			/* These events don't go into final tracefile */
+			if (!(cur->event >= SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV &&
+			    cur->event < SAMPLING_ADDRESS_ALLOCATED_OBJECT_CALLER_EV+MAX_CALLERS))
+				nevents++;
 		}
 		else
 			break;
@@ -753,11 +822,11 @@ static int paraver_build_multi_event (struct fdz_fitxer fdz, paraver_rec_t ** cu
 	}
 
 	paraver_multi_event (fdz, prev_cpu, prev_ptask, prev_task, prev_thread,
-	  prev_time, i, events, values);
+	  prev_time, nevents, events, values);
 
 	*current = cur;
 	if (num_events != NULL)
-		*num_events = i;
+		*num_events = nevents;
 	return 0;
 #undef MAX_EVENT_COUNT_IN_MULTI_EVENT
 }
@@ -991,7 +1060,9 @@ static void Paraver_JoinFiles_Master (int numtasks, PRVFileSet_t *prvfset,
 		{
 			case UNFINISHED_STATE:
 			if (num_incomplete_state == 0)
-				fprintf (stderr, "mpi2prv: Error! Found an unfinished state! Continuing...\n");
+				fprintf (stderr, "mpi2prv: Error! Found an unfinished state in object %d.%d.%d at time %llu (event %llu out of %llu)! Continuing...\n",
+				current->ptask, current->task, current->thread, current->time,
+				current_event, num_of_events);
 			num_incomplete_state++;
 			current = GetNextParaver_Rec (prvfset);
 			current_event++;
@@ -1047,9 +1118,10 @@ static void Paraver_JoinFiles_Master (int numtasks, PRVFileSet_t *prvfset,
 
 		if (pct > last_pct + 5.0 && pct <= 100.0)
 		{
-			fprintf (stdout, "%.1lf%% ", pct);
+			fprintf (stdout, "%d%% ", (int) pct);
 			fflush (stdout);
-			last_pct += 5.0;
+			while (last_pct + 5.0 < pct)
+				last_pct += 5.0;
 		}
 	}
 	while (current != NULL && !error);
