@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/hwc/common_hwc.c $
- | @last_commit: $Date: 2015-04-22 14:39:08 +0200 (mié, 22 abr 2015) $
- | @version:     $Revision: 3263 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: common_hwc.c 3263 2015-04-22 12:39:08Z harald $";
-
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
@@ -54,6 +47,9 @@ static char UNUSED rcsid[] = "$Id: common_hwc.c 3263 2015-04-22 12:39:08Z harald
 #include "xml-parse.h"
 #include "common_hwc.h"
 #include "misc_wrapper.h"
+#if defined(ENABLE_PEBS_SAMPLING)
+# include "sampling-intel-pebs.h"
+#endif
 
 /*------------------------------------------------ Global Variables ---------*/
 int HWCEnabled = FALSE;           /* Have the HWC been started? */
@@ -224,6 +220,7 @@ void HWC_Start_Current_Set (UINT64 countglops, UINT64 time, int thread_id)
  */
 void HWC_Start_Next_Set (UINT64 countglops, UINT64 time, int thread_id)
 {
+
 	/* If there are less than 2 sets, don't do anything! */
 	if (HWC_num_sets > 1)
 	{
@@ -237,6 +234,10 @@ void HWC_Start_Next_Set (UINT64 countglops, UINT64 time, int thread_id)
 
 		HWC_Start_Current_Set (countglops, time, thread_id);
 	}
+
+#if defined(ENABLE_PEBS_SAMPLING) 
+	Extrae_IntelPEBS_nextSampling();
+#endif
 }
 
 /** 
@@ -258,6 +259,10 @@ void HWC_Start_Previous_Set (UINT64 countglops, UINT64 time, int thread_id)
 
 		HWC_Start_Current_Set (countglops, time, thread_id);
 	}
+
+#if defined(ENABLE_PEBS_SAMPLING) 
+	Extrae_IntelPEBS_nextSampling();
+#endif
 }
 
 /** 
