@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/hwc/papi_hwc.c $
- | @last_commit: $Date: 2015-04-22 16:43:20 +0200 (mié, 22 abr 2015) $
- | @version:     $Revision: 3266 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: papi_hwc.c 3266 2015-04-22 14:43:20Z harald $";
-
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
@@ -426,11 +419,11 @@ int HWCBE_PAPI_Start_Set (UINT64 countglops, UINT64 time, int numset, int thread
 			  HWC_sets[numset].OverflowValue[i], 0, PAPI_sampling_handler);
 			if (rc < 0)
 			{
-				setSamplingEnabled (FALSE);
+				Extrae_setSamplingEnabled (FALSE);
 				fprintf (stderr, PACKAGE_NAME": PAPI_overflow failed for thread %d - counter %x!\n", threadid, HWC_sets[numset].OverflowCounter[i]);
 			}
 			else
-				setSamplingEnabled (TRUE);
+				Extrae_setSamplingEnabled (TRUE);
 		}
 	}
 #endif
