@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/interfaces/API/misc_interface.c $
- | @last_commit: $Date: 2015-04-22 15:23:11 +0200 (mié, 22 abr 2015) $
- | @version:     $Revision: 3265 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: misc_interface.c 3265 2015-04-22 13:23:11Z harald $";
-
 #ifdef HAVE_STDIO_H
 # include <stdio.h>
 #endif
@@ -417,14 +410,16 @@ EXPAND_ROUTINE_WITH_PREFIXES(apiTRACE_NEXT_HWC_SET)
 EXPAND_ROUTINE_WITH_PREFIXES(apiTRACE_SETOPTIONS);
 
 #define apiTRACE_USER_FUNCTION(x) \
-	void x##_user_function (unsigned enter) \
+	UINT64 x##_user_function (unsigned enter) \
 	{ \
+		UINT64 r = 0; \
 		if (mpitrace_on) \
 		{ \
 			Backend_Enter_Instrumentation (1); \
-			Extrae_user_function_Wrapper (enter); \
+			r = Extrae_user_function_Wrapper (enter); \
 			Backend_Leave_Instrumentation (); \
 		} \
+		return r; \
 	}
 EXPAND_ROUTINE_WITH_PREFIXES(apiTRACE_USER_FUNCTION);
 
@@ -699,15 +694,17 @@ void Extrae_set_options (int options)
 	}
 }
 
-INTERFACE_ALIASES_C(_user_function, Extrae_user_function,(unsigned enter),void)
-void Extrae_user_function (unsigned enter)
+INTERFACE_ALIASES_C(_user_function, Extrae_user_function,(unsigned enter),UINT64)
+UINT64 Extrae_user_function (unsigned enter)
 {
+	UINT64 r = 0;
 	if (mpitrace_on)
 	{
 		Backend_Enter_Instrumentation (1);
-		Extrae_user_function_Wrapper (enter);
+		r = Extrae_user_function_Wrapper (enter);
 		Backend_Leave_Instrumentation ();
 	}
+	return r;
 }
 
 INTERFACE_ALIASES_C(_function_from_address,Extrae_function_from_address,(extrae_type_t type, void *address),void)
