@@ -21,16 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/API/buffers.c $
- | @last_commit: $Date: 2014-09-08 12:11:01 +0200 (lun, 08 sep 2014) $
- | @version:     $Revision: 2955 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
-
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: buffers.c 2955 2014-09-08 10:11:01Z gllort $";
-
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
@@ -84,7 +76,9 @@ Buffer_t * new_Buffer (int n_events, char *file, int enable_cache)
 {
 	Buffer_t *buffer = NULL;
 #if defined(HAVE_ONLINE)
+#if 0
 	pthread_mutexattr_t attr;
+#endif
 	int rc;
 #endif
 
