@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/OMP/gnu-libgomp-4.2.c $
- | @last_commit: $Date: 2015-06-05 17:18:47 +0200 (vie, 05 jun 2015) $
- | @version:     $Revision: 3354 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: gnu-libgomp-4.2.c 3354 2015-06-05 15:18:47Z harald $";
-
 #ifdef HAVE_DLFCN_H
 # define __USE_GNU
 # include <dlfcn.h>
