@@ -21,11 +21,6 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/OMP/intel-kmpc-11.c $
- | @last_commit: $Date: 2015-04-20 16:32:24 +0200 (lun, 20 abr 2015) $
- | @version:     $Revision: 3250 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
 #ifdef HAVE_DLFCN_H
@@ -53,8 +48,6 @@
 
 //#define DEBUG
 
-static char UNUSED rcsid[] = "$Id: intel-kmpc-11.c 3250 2015-04-20 14:32:24Z harald $";
-
 #if defined(PIC)
 
 #define INC_IF_NOT_NULL(ptr,cnt) (cnt = (ptr == NULL)?cnt:cnt+1)
