@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/pthread/pthread_wrapper.c $
- | @last_commit: $Date: 2015-03-10 17:16:23 +0100 (mar, 10 mar 2015) $
- | @version:     $Revision: 3160 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: pthread_wrapper.c 3160 2015-03-10 16:16:23Z harald $";
-
 #ifdef HAVE_PTHREAD_H
 # include <pthread.h>
 #endif
@@ -52,7 +45,7 @@ static char UNUSED rcsid[] = "$Id: pthread_wrapper.c 3160 2015-03-10 16:16:23Z h
 #include "trace_macros.h"
 #include "pthread_probe.h"
 
-//#define DEBUG
+// #define DEBUG
 //#define DEBUG_MUTEX
 
 #if defined(PIC)
@@ -237,8 +230,6 @@ static void * pthread_create_hook (void *p1)
 
 	Backend_SetpThreadIdentifier (i->pthreadID);
 
-	/* incialitzar hwc, mode (detail/burst), primers events (tempsinit?) */
-
 	/* Notify the calling thread */
 	pthread_mutex_lock_real (&(i->lock));
 	pthread_cond_signal (&(i->wait));
@@ -250,8 +241,14 @@ static void * pthread_create_hook (void *p1)
 
 	res = routine (arg);
 
-	Probe_pthread_Function_Exit ();
-	Backend_Leave_Instrumentation ();
+	/* Another thread may have called Extrae_fini() while executing
+	   the routine thread */
+	if (Extrae_is_initialized_Wrapper() != EXTRAE_NOT_INITIALIZED)
+	{
+		Probe_pthread_Function_Exit ();
+		Backend_Leave_Instrumentation ();
+		Backend_Flush_pThread (pthread_self());
+	}
 
 	return res;
 }
@@ -271,7 +268,7 @@ int pthread_create (pthread_t* p1, const pthread_attr_t* p2,
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_create_real at %p\n", pthread_create_real);
 #endif
 
-	if (pthread_create_real != NULL && EXTRAE_INITIALIZED())
+	if (pthread_create_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_tracing())
 	{
 		/* This is a bit tricky.
 		   Some OSes (like FreeBSD) delegates the pthread library initialization
@@ -327,7 +324,7 @@ int pthread_create (pthread_t* p1, const pthread_attr_t* p2,
 		/* Stop protecting the region, more pthread creations can enter */
 		pthread_mutex_unlock_real (&extrae_pthread_create_mutex);
 	}
-	else if (pthread_create_real != NULL && !EXTRAE_INITIALIZED())
+	else if (pthread_create_real != NULL)
 	{
 		res = pthread_create_real (p1, p2, p3, p4);
 	}
@@ -351,18 +348,24 @@ int pthread_join (pthread_t p1, void **p2)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_join (%p, %p)\n", p1, p2);
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_join_real at %p\n", pthread_join_real);
 #endif
-	if (pthread_join_real != NULL && EXTRAE_INITIALIZED())
+
+	if (pthread_join_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_tracing())
 	{
 		Backend_Enter_Instrumentation (2);
 		Probe_pthread_Join_Entry ();
 
 		res = pthread_join_real (p1, p2);
-		Backend_Flush_pThread (p1);
 
-		Probe_pthread_Join_Exit ();
-		Backend_Leave_Instrumentation ();
+		/* Careful, initialized state may have changed after join! */
+		if (Extrae_is_initialized_Wrapper())
+		{
+			Backend_Flush_pThread (p1);
+
+			Probe_pthread_Join_Exit ();
+			Backend_Leave_Instrumentation ();
+		}
 	}
-	else if (pthread_join_real != NULL && !EXTRAE_INITIALIZED())
+	else if (pthread_join_real != NULL)
 	{
 		res = pthread_join_real (p1, p2);
 	}
@@ -384,7 +387,7 @@ void pthread_exit (void *p1)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_exit_real at %p\n", pthread_exit_real);
 #endif
 
-	if (pthread_exit_real != NULL && EXTRAE_INITIALIZED())
+	if (pthread_exit_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_tracing())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -392,13 +395,12 @@ void pthread_exit (void *p1)
 			Probe_pthread_Function_Exit();
 			Probe_pthread_Exit_Entry();
 			Backend_Leave_Instrumentation ();
-
 			Backend_Flush_pThread (pthread_self());
 		}
 
 		pthread_exit_real (p1);
 	}
-	else if (pthread_exit_real != NULL && !EXTRAE_INITIALIZED())
+	else if (pthread_exit_real != NULL)
 	{
 		pthread_exit_real (p1);
 	}
@@ -421,7 +423,7 @@ int pthread_detach (pthread_t p1)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_detach_real at %p\n", pthread_detach_real);
 #endif
 
-	if (pthread_detach_real != NULL && EXTRAE_INITIALIZED())
+	if (pthread_detach_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_tracing())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -432,7 +434,7 @@ int pthread_detach (pthread_t p1)
 			Backend_Leave_Instrumentation ();
 		}
 	}
-	else if (pthread_detach_real != NULL && !EXTRAE_INITIALIZED())
+	else if (pthread_detach_real != NULL)
 	{
 		res = pthread_detach_real (p1);
 	}
@@ -457,7 +459,8 @@ int pthread_mutex_lock (pthread_mutex_t *m)
 #endif
 
 	/* Caution! pthread_exit() seems to call pthread_mutex_lock */
-	if (pthread_mutex_lock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_mutex_lock_real != NULL && EXTRAE_INITIALIZED()
+	  && Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -492,7 +495,8 @@ int pthread_mutex_trylock (pthread_mutex_t *m)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_mutex_trylock_real at %p\n", pthread_mutex_trylock_real);
 #endif
 
-	if (pthread_mutex_trylock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_mutex_trylock_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -528,7 +532,8 @@ int pthread_mutex_timedlock(pthread_mutex_t *m, const struct timespec *t)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_mutex_timedlock_real at %p\n", pthread_mutex_timedlock_real);
 #endif
 
-	if (pthread_mutex_timedlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_mutex_timedlock_real != NULL && EXTRAE_INITIALIZED() && 
+	 Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -565,7 +570,8 @@ int pthread_mutex_unlock (pthread_mutex_t *m)
 #endif
 
 	/* Caution! pthread_exit() seems to call pthread_mutex_lock */
-	if (pthread_mutex_unlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_mutex_unlock_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -603,7 +609,8 @@ int pthread_cond_signal (pthread_cond_t *c)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_cond_signal_real at %p\n", pthread_cond_signal_real);
 #endif
 
-	if (pthread_cond_signal_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_cond_signal_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -639,7 +646,8 @@ int pthread_cond_broadcast (pthread_cond_t *c)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_cond_broadcast_real at %p\n", pthread_cond_broadcast_real);
 #endif
 
-	if (pthread_cond_broadcast_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_cond_broadcast_real != NULL && EXTRAE_INITIALIZED() &&
+	   Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -674,7 +682,8 @@ int pthread_cond_wait (pthread_cond_t *c, pthread_mutex_t *m)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_cond_wait_real at %p\n", pthread_cond_wait_real);
 #endif
 
-	if (pthread_cond_wait_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_cond_wait_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -709,7 +718,8 @@ int pthread_cond_timedwait (pthread_cond_t *c, pthread_mutex_t *m, const struct
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_cond_wait_real at %p\n", pthread_cond_wait_real);
 #endif
 
-	if (pthread_cond_timedwait_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_cond_timedwait_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -746,7 +756,8 @@ int pthread_rwlock_rdlock (pthread_rwlock_t *l)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_rwlock_rdlock_real at %p\n", pthread_rwlock_rdlock_real);
 #endif
 
-	if (pthread_rwlock_rdlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_rwlock_rdlock_real != NULL && EXTRAE_INITIALIZED() &&
+	 Extrae_get_pthread_tracing() &&  Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -782,7 +793,8 @@ int pthread_rwlock_tryrdlock(pthread_rwlock_t *l)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_rwlock_tryrdlock_real at %p\n", pthread_rwlock_tryrdlock_real);
 #endif
 
-	if (pthread_rwlock_tryrdlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_rwlock_tryrdlock_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -818,7 +830,8 @@ int pthread_rwlock_timedrdlock(pthread_rwlock_t *l, const struct timespec *t)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_rwlock_timedrdlock_real at %p\n", pthread_rwlock_timedrdlock_real);
 #endif
 
-	if (pthread_rwlock_timedrdlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_rwlock_timedrdlock_real != NULL && EXTRAE_INITIALIZED() &&
+	 Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -854,7 +867,8 @@ int pthread_rwlock_wrlock(pthread_rwlock_t *l)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_rwlock_wrlock_real at %p\n", pthread_rwlock_wrlock_real);
 #endif
 
-	if (pthread_rwlock_wrlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_rwlock_wrlock_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -889,7 +903,8 @@ int pthread_rwlock_trywrlock(pthread_rwlock_t *l)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_rwlock_trywrlock_real at %p\n", pthread_rwlock_trywrlock_real);
 #endif
 
-	if (pthread_rwlock_trywrlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_rwlock_trywrlock_real != NULL && EXTRAE_INITIALIZED() &&
+	 Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -924,7 +939,8 @@ int pthread_rwlock_timedwrlock(pthread_rwlock_t *l, const struct timespec *t)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_rwlock_timedwrlock_real at %p\n", pthread_rwlock_timedwrlock_real);
 #endif
 
-	if (pthread_rwlock_timedwrlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_rwlock_timedwrlock_real != NULL && EXTRAE_INITIALIZED() &&
+	 Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -960,7 +976,8 @@ int pthread_rwlock_unlock(pthread_rwlock_t *l)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_rwlock_unlock_real at %p\n", pthread_rwlock_unlock_real);
 #endif
 
-	if (pthread_rwlock_unlock_real != NULL && EXTRAE_INITIALIZED() && Extrae_get_pthread_instrument_locks())
+	if (pthread_rwlock_unlock_real != NULL && EXTRAE_INITIALIZED() &&
+	 Extrae_get_pthread_tracing() && Extrae_get_pthread_instrument_locks())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -996,7 +1013,8 @@ int pthread_barrier_wait (pthread_barrier_t *barrier)
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_barrier_wait (%p)\n", barrier);
 	fprintf (stderr, PACKAGE_NAME": DEBUG: pthread_barrier_wait_real at %p\n", pthread_barrier_wait_real);
 #endif
-	if (pthread_barrier_wait_real != NULL && EXTRAE_INITIALIZED())
+	if (pthread_barrier_wait_real != NULL && EXTRAE_INITIALIZED() &&
+	  Extrae_get_pthread_tracing())
 	{
 		if (!Backend_ispThreadFinished(THREADID))
 		{
@@ -1007,7 +1025,7 @@ int pthread_barrier_wait (pthread_barrier_t *barrier)
 			Backend_Leave_Instrumentation ();
 		}
 	}
-	else if (pthread_barrier_wait_real != NULL && !EXTRAE_INITIALIZED())
+	else if (pthread_barrier_wait_real != NULL)
 	{
 		res = pthread_barrier_wait_real (barrier);
 	}
